# -*- coding: utf-8 -*-

# --------------------------------------------------------------------------
# Copyright Commvault Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# --------------------------------------------------------------------------

"""Helper file to maintain all the constants for threat indicators

ThreatConstants         --  class to maintains constants for Threat scan and threat analysis

FileTypeConstants       --  class to maintain constants for file type anomalies

RequestConstants        --  class to maintain request json for threat indicators related operations

"""


class ThreatConstants:
    """class to maintains constants for Threat scan and threat analysis"""
    FIELD_INFECTED_COUNT = 'infectedFilesCount'
    FIELD_FINGERPRINT_COUNT = 'fingerPrintFilesCount'
    FIELD_DATASOURCE_ID = 'dataSourceId'


class FileTypeConstants:
    """class to maintain constants for file type anomalies"""
    FIELD_DELETE_COUNT = 'deleteCount'
    FIELD_RENAME_COUNT = 'renameCount'
    FIELD_CREATE_COUNT = 'createCount'
    FIELD_MODIFIED_COUNT = 'modCount'


class RequestConstants:
    """class to maintain request json for threat indicator related operations"""
    CLEAR_ANOMALY_JSON = {"clients": [{"_type_": "CLIENT_ENTITY",
                                       "clientId": 0,
                                       "displayName": "",
                                       "dataSourceId": 0,
                                       "selected": True}],
                          "anomalyTypes": []}

    RUN_SCAN_JSON = {"client": {
        "clientId": 0},
        "timeRange": {"fromTime": 0, "toTime": 0},
        "threatAnalysisFlags": 0,
        "indexServer": {"clientId": 0},
        "backupDetails": [{"copyId": 0, "storagePoolId": 0}]}



# --------------------------------------------------------------------------
# Copyright Commvault Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# --------------------------------------------------------------------------

"""File for performing index server related operations on the commcell

IndexServers, IndexServer and _Roles are 3 classes defined in this file

IndexServers:   Class for representing all the index servers associated with the commcell

IndexServer:    Class for a instance of a single index server of the commcell

_Roles:         Class for storing all the cloud role details

"IndexServerOSType" is the enum class used to represent os type of IS

IndexServers
============

    __init__()                          --  initialize object of IndexServers class associated with
                                            the commcell

    __str()                             --  returns all the index servers of the commcell

    __repr__()                          --  returns the string to represent the instance

    __len__()                           --  returns the number of index servers associated

    _get_index_servers()                --  gets all the index server associated with the commcell

    _response_not_success()             --  raise exception when response is not 200

    _get_all_roles()                    --  creates an instance of _Roles class

    has()                               --  returns whether the index server is present or not

    get()                               --  returns a IndexServer object for given cloud name

    create()                            --  creates a index server within the commcell

    delete()                            --  deletes a index server associated with commcell

    update_roles_data()                 --  fetches the cloud roles data from commcell

    get_properties()                    --  returns a dict of data of index server for the given
                                            cloud name

    refresh()                           --  refresh the index servers associated with commcell

    prune_orphan_datasources()          --  Deletes all the orphan datasources

IndexServers Attributes
-----------------------

    **all_index_servers**               --  returns the dictionary consisting of all the index
                                            servers associated with the commcell and there details

    **roles_data**                      --  returns the list of cloud roles details


IndexServer
===========

    __init()__                          --  initializes the object with the specified commcell
                                            object, index server name and the cloud id

    __repr__()                          --  returns the index server's name, the instance is
    associated with

    _get_cloud_id()                     --  gets the cloud id

    _get_properties()                   --  gets all the properties of the index server

    refresh()                           --  refresh all the properties of client

    update_roles_data()                 --  fetches the cloud roles data from commcell

    modify()                            --  to modify the index server node details

    change_plan()                       --  changes the plan of a given index server

    update_role()                       --  to update the roles assigned to cloud

    delete_docs_from_core()             --  Deletes the docs from the given core name on index server depending
                                            on the select dict passed

    hard_commit                         --  do hard commit on specified index server solr core

    get_health_indicators()             --  get health indicators for index server node by client name

    get_all_cores                       --  gets all the cores in index server

    _create_solr_query()                --  Create solr search query based on inputs provided

    execute_solr_query()                --  Creates solr url based on input and executes it on solr on given core

    get_index_node()                    --  returns an Index server node object for given node name

    get_os_info()                       --  returns the OS type for the Index server

    get_plan_info()                     --  Returns the plan information of the index server

    __form_field_query()                --  returns the query with the key and value passed

IndexServer Attributes
----------------------

    **properties**                      --  returns the properties of this index server

    **roles_data**                      --  returns all the available cloud roles data

    **host_name**                       --  returns the host name for the index server

    **internal_cloud_name**             --  returns the internal cloud name

    **client_name**                     --  returns the client name for index server

    **server_url**                      --  returns the content indexing server url

    **type**                            --  returns the type of the index server

    **base_port**                       --  returns the base port of this index server

    **client_id**                       --  returns the client id for this index server

    **roles**                           --  returns the array of roles installed
                                            with the index server within the commcell

    **cloud_id**                        --  returns the cloud id of the index server

    **server_type**                     --  returns the server type of the index server

    **engine_name**                     --  returns the engine name that is index server name

    **index_server_client_id**          --  returns the index server client id

    **role_display_name**               --  display name of roles

    **is_cloud**                        --  returns boolean True if the Index server is cloud else returns False

    **node_count**                      --  returns the number of Index server nodes

    **os_info**                         --  returns the OS type for the Index server

    **plan_name**                       --  Returns the plan name associated with index server

    **fs_collection**                   --  Returns the multinode collection name of File System Index


IndexNode
=========

    __init__()                          --  initializes the class with commcell object
                                            Index server cloud id and Node client name

    refresh()                           --  refreshes the attributes

    modify()                            --  to modify the index server node details

IndexNode Attributes
--------------------

    **node_name**                       --  returns Index server node client name

    **node_id**                         --  returns Index server node client id

    **solr_port**                       --  returns port number Solr is running on the\
                                            Index server node

    **solr_url**                        --  returns Solr URL for Index server node

    **roles**                           --  returns the array of roles installed
                                            with the index server within the commcell

    **index_location**                  --  returns Index directory for the Index server Node

    **jvm_memory**                      --  returns Solr JVM memory for the Index server Node

_Roles
======

    __init__()                          --  initializes the class with commcell object

    refresh()                           --  refreshes the attributes

    _get_all_roles()                    --  fetches the cloud roles data from commcell

    get_role_id()                       --  returns role id for given role name

    update_roles_data()                 --  fetches the cloud roles data from commcell

_Roles Attributes
-----------------

    **roles_data**                      --  returns the list of details of all cloud roles
    """
import json

import http.client as httplib
from copy import deepcopy
import enum
from .exception import SDKException
from .datacube.constants import IndexServerConstants


class IndexServers(object):
    """Class for representing all the index servers associated with the commcell."""

    def __init__(self, commcell_object):
        """Initialize object of the IndexServers class.

            Args:
                commcell_object (object)  --  instance of the Commcell class

            Returns:
                object - instance of the IndexServers class
        """
        self._commcell_object = commcell_object
        self._cvpysdk_object = commcell_object._cvpysdk_object
        self._services = commcell_object._services
        self._update_response_ = commcell_object._update_response_
        self._all_index_servers = None
        self._roles_obj = None
        self.refresh()

    def __str__(self):
        """Representation string consisting of all index servers of the commcell.

                Returns:
                    str - string of all the index servers with different roles associated
                    with the commcell
        """
        representation_string = '{:^5}\t{:^20}\n\n'.format('S. No.', 'IS Name')
        index = 1
        for index_server in self._all_index_servers:
            representation_string += '{:^5}\t{:^20}\n'.format(
                index, index_server['engineName'])
            index += 1
        return representation_string

    def __repr__(self):
        """Representation string for the instance of the IndexServers class."""
        return "IndexServers class instance for Commcell: '{0}'".format(
            self._commcell_object.commserv_name
        )

    def __len__(self):
        """Returns the number of the index servers associated with the commcell"""
        return len(self._all_index_servers)

    def _response_not_success(self, response):
        """Helper method to raise exception when response is not 200 (ok)

                Raises:
                    SDKException:
                        Response was not success
        """
        raise SDKException(
            'Response',
            '101',
            self._update_response_(
                response.text))

    def _get_index_servers(self):
        """Method to retrieve all the index server available on commcell.

            Raises:
                SDKException:
                    Failed to get the list of analytics engines

                    Response was not success
        """
        flag, response = self._cvpysdk_object.make_request(
            'GET', self._services['GET_ALL_INDEX_SERVERS'])
        if flag:
            if response.json() and 'listOfCIServer' in response.json():
                for item in response.json()['listOfCIServer']:
                    if item['cloudID'] in self._all_index_servers:
                        # Add only unique roles to list
                        if 'version' in item and item['version'] not in self._all_index_servers[item['cloudID']]['version']:
                            self._all_index_servers[item['cloudID']]['version'].append(item['version'])
                        # check whether we have populated node details earlier. if not, add it to
                        # exisitng respective fields
                        if item['clientName'] not in self._all_index_servers[item['cloudID']]['clientName']:

                            self._all_index_servers[item['cloudID']]['clientId'].append(item['clientId'])
                            self._all_index_servers[item['cloudID']]['clientName'].append(item['clientName'])
                            self._all_index_servers[item['cloudID']]['hostName'].append(item['hostName'])
                            self._all_index_servers[item['cloudID']]['cIServerURL'].append(item['cIServerURL'])
                            self._all_index_servers[item['cloudID']]['basePort'].append(item['basePort'])

                    else:
                        item['version'] = [item.get('version', '')]
                        item['clientId'] = [item['clientId']]
                        item['clientName'] = [item['clientName']]
                        item['hostName'] = [item['hostName']]
                        item['cIServerURL'] = [item['cIServerURL']]
                        item['basePort'] = [item['basePort']]
                        self._all_index_servers[item['cloudID']] = item
            else:
                self._all_index_servers = {}
        else:
            self._response_not_success(response)

    def _get_all_roles(self):
        """Creates an instance of _Roles class and adds it to the IndexServer class"""
        self._roles_obj = _Roles(self._commcell_object)

    @property
    def all_index_servers(self):
        """Returns the details of all the index server for associated commcell.

                Returns:
                    dict - dictionary consisting details of all the index servers
                    associated with commcell
                    Sample - {
                                <cloud_id_1>   :
                                    {
                                        "engineName" : <property_value>,
                                        "internalCloudName" : <property_value>,
                                        ...
                                    },
                                <cloud_id_2>   :
                                    {
                                        "engineName" : <property_value>,
                                        "cloudID" : <property_value>,
                                        ...
                                    }
                            }
        """
        return self._all_index_servers

    @property
    def roles_data(self):
        """Returns the details of all the cloud roles data

                Returns:
                    list - list of dictionary containing details of the cloud roles
        """
        return self._roles_obj.roles_data

    def refresh(self):
        """Refresh the properties of IndexServers class"""
        self._all_index_servers = {}
        self._get_index_servers()
        if not self._roles_obj:
            self._get_all_roles()

    def update_roles_data(self):
        """Synchronises all the cloud roles details with the commcell"""
        self._roles_obj.update_roles_data()

    def get_properties(self, cloud_name):
        """Returns all details of a index server with the cloud name

                Args:
                    cloud_name     (str)       --  cloud name of index server

                Returns:
                    dict        -   dict consisting details of the index server
        """
        for index_server in self._all_index_servers:
            if self._all_index_servers[index_server]['engineName'] == cloud_name:
                return self._all_index_servers[index_server]
        raise SDKException('IndexServers', '102')

    def has(self, cloud_name):
        """Returns True if the index server with given name is present in commcell.

                Args:
                    cloud_name     (str)       --  the engine name of index server

                Returns:
                    boolean     -   True if index server with given name as is_name
                    is associated with the commcell else returns False

                Raises:
                    SDKExecption:
                        Data type of the input(s) is not valid
        """
        if isinstance(cloud_name, str):
            for index_server in self._all_index_servers:
                if self._all_index_servers[index_server]["engineName"].lower() == cloud_name.lower():
                    return True
            return False
        raise SDKException('IndexServers', '101')

    def get(self, cloud_data):
        """Returns IndexServer object if a index server is found.

                Args:
                    cloud_data        (int/str)       --    cloud name or
                                                            cloud ID of index server

                Returns:
                    object            (IndexServer)   --  Instance on index server with
                    the engine name or cloud id as item

                Raises:
                    SDKException:
                        Index Server not found.

                        Data type of the input(s) is not valid.
        """
        if isinstance(cloud_data, int):
            if cloud_data in self._all_index_servers:
                return IndexServer(
                    self._commcell_object,
                    self._all_index_servers[cloud_data]['engineName'],
                    cloud_data)
            SDKException('IndexServers', '102')
        elif isinstance(cloud_data, str):
            name = cloud_data.lower()
            for itter in self._all_index_servers:
                if self._all_index_servers[itter]['engineName'].lower(
                ) == name:
                    return IndexServer(
                        self._commcell_object,
                        self._all_index_servers[itter]['engineName'],
                        self._all_index_servers[itter]['cloudID'])
            raise SDKException('IndexServers', '102')
        raise SDKException('IndexServers', '101')

    def create(
            self,
            index_server_name,
            index_server_node_names,
            index_directory,
            index_server_roles,
            index_pool_name=None,
            is_cloud=False,
            cloud_param=None):
        """Creates an index server within the commcell

                Args:
                    index_server_node_names         (list)  --  client names for index server node
                    index_server_name               (str)   --  name for the index server
                    index_directory                 (list)  --  list of index locations for the index server
                                                                nodes respectively
                                                    For example:
                                                            [<path_1>] - same index location for all the nodes
                                                            [<path_1>, <path_2>, <path_3>] - different index
                                                    location for index server with 3 nodes
                    index_server_roles              (list)  --  list of role names to be assigned
                    index_pool_name                 (str)   --  name for the index pool to used by cloud index server
                    cloud_param                     (list)  --  list of custom parameters to be parsed
                                                    into the json for index server meta info
                                                    [
                                                        {
                                                            "name": <name>,
                                                            "value": <value>
                                                        }
                                                    ]
                    is_cloud            (bool)  --  if true then creates a cloud mode index server

                Raises:
                    SDKException:
                        Data type of the input(s) is not valid.

                        Response was not success.

                        Response was empty.
        """
        if not (isinstance(index_server_roles, list) and isinstance(index_server_node_names, list)
                and isinstance(index_server_name, str)):
            raise SDKException('IndexServers', '101')
        if isinstance(index_directory, str):
            index_directory = index_directory.split(",")
        node_count = len(index_server_node_names)
        index_directories_count = len(index_directory)
        if index_directories_count != 1 and index_directories_count != node_count:
            raise SDKException('IndexServers', '101')
        cloud_meta_infos = {
            'REPLICATION': '1',
            'LANGUAGE': '0'
        }
        node_meta_infos = {
            'PORTNO': IndexServerConstants.DEFAULT_SOLR_PORT,
            'JVMMAXMEMORY': IndexServerConstants.DEFAULT_JVM_MAX_MEMORY
        }
        role_meta_infos = {}
        req_json = deepcopy(IndexServerConstants.REQUEST_JSON)
        req_json['cloudInfoEntity'] = {
            'cloudName': index_server_name,
            'cloudDisplayName': index_server_name
        }
        if is_cloud:
            index_pool_obj = self._commcell_object.index_pools[index_pool_name]
            req_json['type'] = 5
            req_json['solrCloudInfo']['cloudPoolInfo'] = {
                'cloudId': int(index_pool_obj['pool_id'])
            }
            cloud_meta_infos['INDEXLOCATION'] = index_directory[0]
        for node_name_index in range(len(index_server_node_names)):
            node_name = index_server_node_names[node_name_index]
            location_index = node_name_index - (node_name_index//index_directories_count)
            node_obj = self._commcell_object.clients[node_name]
            node_data = {
                "opType": IndexServerConstants.OPERATION_ADD,
                "nodeClientEntity": {
                    "hostName": node_obj['hostname'],
                    "clientId": int(node_obj['id']),
                    "clientName": node_name
                },
                'nodeMetaInfos': [{
                    "name": "INDEXLOCATION",
                    "value": index_directory[location_index]
                }]
            }
            for node_info in node_meta_infos:
                node_data['nodeMetaInfos'].append({
                    'name': node_info,
                    'value': str(node_meta_infos[node_info])
                })
            req_json['cloudNodes'].append(node_data)
        for role in index_server_roles:
            role_id = self._roles_obj.get_role_id(role)
            if not role_id:
                raise SDKException('IndexServers', '103')
            role_data = {
                "roleId": role_id,
                "roleName": role,
                "operationType": IndexServerConstants.OPERATION_ADD,
                'roleMetaInfos': []
            }
            for role_info in role_meta_infos:
                role_data['roleMetaInfos'].append({
                    'name': role_info,
                    'value': role_meta_infos[role_info]
                })
            req_json['solrCloudInfo']['roles'].append(role_data)
        if cloud_param:
            for param in cloud_param:
                if param['name'] in cloud_meta_infos:
                    del cloud_meta_infos[param['name']]
                req_json['cloudMetaInfos'].append(param)
        for cloud_info in cloud_meta_infos:
            req_json['cloudMetaInfos'].append({
                'name': cloud_info,
                'value': cloud_meta_infos[cloud_info]
            })
        flag, response = self._cvpysdk_object.make_request(
            'POST', self._services['CLOUD_CREATE'], req_json)
        if flag:
            if response.json():
                error_code = response.json()['genericResp']['errorCode']
                error_string = response.json()['genericResp']['errorMessage']
                if error_code == 0:
                    self.refresh()
                    self._commcell_object.clients.refresh()
                    self._commcell_object.datacube.refresh_engine()
                else:
                    o_str = 'Failed to create Index Server. Error: "{0}"'.format(
                        error_string)
                    raise SDKException('IndexServers', '102', o_str)
            else:
                raise SDKException('Response', '102')
        else:
            self._response_not_success(response)

    def delete(self, cloud_name):
        """Deletes / removes an index server from the commcell

                Args:
                    cloud_name      (str)   --  cloud name of index server
                    to be removed from the commcell

                Raises:
                    SDKException:
                        Data type of the input(s) is not valid.

                        Response was not success.

                        Response was empty.
        """
        if not isinstance(cloud_name, str):
            raise SDKException('IndexServers', '101')
        cloud_id = self.get(cloud_name).cloud_id
        req_json = deepcopy(IndexServerConstants.REQUEST_JSON)
        req_json["opType"] = IndexServerConstants.OPERATION_DELETE
        req_json['cloudInfoEntity']['cloudId'] = cloud_id
        flag, response = self._cvpysdk_object.make_request(
            'POST', self._services['CLOUD_DELETE'], req_json
        )
        if flag:
            if response.json() and 'genericResp' in response.json() \
                    and 'errorCode' not in response.json()['genericResp']:
                self.refresh()
                self._commcell_object.clients.refresh()
                self._commcell_object.datacube.refresh_engine()
                return
            if response.json() and 'genericResp' in response.json():
                raise SDKException(
                    'Response', '102', response.json()['genericResp'].get(
                        'errorMessage', ''))
            raise SDKException('Response', '102')
        self._response_not_success(response)

    def prune_orphan_datasources(self):
        """Deletes all the orphan datasources
            Raises:
                SDKException:
                    if failed to prune the orphan datasources

                    If response is empty

                    if response is not success
        """
        prune_datasource = self._services['PRUNE_DATASOURCE']
        request_json = IndexServerConstants.PRUNE_REQUEST_JSON
        flag, response = self._cvpysdk_object.make_request(
            'POST', prune_datasource, request_json)
        if flag:
            if response.json():
                error_code = response.json().get('errorCode', 0)
                if error_code != 0:
                    raise SDKException('IndexServers', '104', 'Failed to prune orphan datasources')
                return
            raise SDKException('Response', '102')
        raise SDKException('Response', '101', self._update_response_(response.text))


class IndexServerOSType(enum.Enum):
    """Enum class for Index Server OS Type"""
    WINDOWS = "Windows"
    UNIX = "Unix"
    MIXED = "Mixed"


class IndexServer(object):
    """Class for performing index server operations for a specific index server"""

    def __init__(self, commcell_obj, name, cloud_id=None):
        """Initialize the IndexServer class instance.

            Args:
                commcell_obj    (object)        --  instance of the Commcell class

                name            (str)           --  name of the index server

                cloud_id        (int)           --  cloud id of the index server
                    default: None

            Returns:
                object - instance of the IndexServer class
        """
        self._engine_name = name
        self._commcell_obj = commcell_obj
        self._cvpysdk_object = self._commcell_obj._cvpysdk_object
        self._services = self._commcell_obj._services
        if cloud_id:
            self._cloud_id = cloud_id
        else:
            self._cloud_id = self._get_cloud_id()
        self._properties = None
        self._roles_obj = None
        self.plan_info = None
        self.os_type = None
        self.refresh()

    def __repr__(self):
        """String representation of the instance of this class."""
        return 'IndexServer class instance for index server: "{0}"'.format(
            self._engine_name)

    def _get_cloud_id(self):
        """Get the cloud id for the index server

                Returns:
                    int - cloud id for the index server
        """
        return self._commcell_obj.index_servers.get(self._engine_name).cloud_id

    def _get_properties(self):
        """Get the properties of the index server"""
        self._properties = self._commcell_obj.index_servers.get_properties(
            self._engine_name)

    def refresh(self):
        """Refresh the index server properties"""
        self._commcell_obj.index_servers.refresh()
        self._get_properties()
        if self.os_type is None:
            self.os_type = self.get_os_info()
        if not self._roles_obj:
            self._roles_obj = _Roles(self._commcell_obj)
        if self.plan_info is None:
            self.plan_info = self.get_plan_info()

    def update_roles_data(self):
        """Synchronize the cloud roles data with the commcell"""
        self._roles_obj.update_roles_data()

    def modify(self, index_location, node_name, node_params):
        """Modifies the properties of an index server

            Args:
                index_location      (str)       --  index server data directory
                node_name           (str)       --  index server node name
                node_params         (dict)      --  parameters to be passed
                                                    [
                                                        {
                                                            "name" : <property_name>,
                                                            "value" : <property_value>
                                                        }
                                                    ]
            Raises:
                SDKException:
                    Response was not success.
                    Response was empty.
        """
        json_req = deepcopy(IndexServerConstants.REQUEST_JSON)
        json_req['opType'] = IndexServerConstants.OPERATION_EDIT
        json_req['cloudNodes'] = [{
            "opType": IndexServerConstants.OPERATION_EDIT,
            "nodeClientEntity": {
                "clientId": int(self._commcell_obj.clients.get(node_name).client_id)
            },
            "nodeMetaInfos": [
                {
                    "name": "INDEXLOCATION",
                    "value": index_location
                }
            ]
        }]
        json_req['cloudInfoEntity']['cloudId'] = self.cloud_id
        for param in node_params:
            json_req['cloudNodes'][0]['nodeMetaInfos'].append(param)
        flag, response = self._cvpysdk_object.make_request(
            "POST", self._services['CLOUD_MODIFY'], json_req)
        if flag:
            if response.json():
                if 'cloudId' in response.json():
                    self.refresh()
                    return
            raise SDKException('Response', '102')
        raise SDKException('Response', '101')

    def change_plan(self, plan_name):
        """Modifies the plan used by an index server

            Args:
                plan_name      (str)       --  Name of the plan to be used for the index server
            Raises:
                SDKException:
                    Response was not success.
                    Response was empty.
                    if plan with given name doesn't exist
        """
        if not self._commcell_obj.plans.has_plan(plan_name):
            raise SDKException(
                'Plan', '102', f"Plan with name [{plan_name}] doesn't exist")
        request_json = {
            "opType": IndexServerConstants.OPERATION_EDIT,
            "type": 1,
            "planInfo": {
                "planId": int(self._commcell_obj.plans.get(plan_name).plan_id)
            },
            "cloudInfoEntity": {
                "cloudId": self.cloud_id
            }
        }
        flag, response = self._cvpysdk_object.make_request(
            "POST", self._services['CLOUD_MODIFY'], request_json)
        if flag:
            if response.json():
                if 'cloudId' in response.json():
                    self.refresh()
                    return
            raise SDKException('Response', '102')
        raise SDKException('Response', '101')

    def update_role(self, props=None):
        """Updates a role of an Index Server

            Args:
                props               (list)  --  array of dictionaries
                consisting details of the roles such as role name
                and operation type.
                                            [{
                                                "roleName": <name>          (str)
                                                "operationType": 1 or 2     (int)
                                                    1 for adding a role
                                                    2 for removing a role
                                            }]

            Raises:
                SDKException:
                    if response is empty

                    if response is not success

        """
        json_req = {"cloudId": self.cloud_id, "roles": []}
        if props:
            for prop in props:
                role_id = self._roles_obj.get_role_id(prop['roleName'])
                if not role_id:
                    raise SDKException('IndexServers', '103')
                prop['roleId'] = role_id
                json_req['roles'].append(prop)
        flag, response = self._cvpysdk_object.make_request(
            'POST', self._services['CLOUD_ROLE_UPDATE'], json_req
        )
        if flag:
            if response.json() and 'errorCode' in response.json():
                if response.json()['errorCode'] == 0:
                    self.refresh()
                    return
            raise SDKException('Response', '102')
        raise SDKException('Response', '101')

    def delete_docs_from_core(self, core_name, select_dict = None):
        """Deletes the docs from the given core name on index server depending on the select dict passed

                Args:

                        core_name               (str)  --  name of the solr core
                        select_dict             (dict) --  dict with query to delete specific documents
                                                    default query - "*:*" (Deletes all the docs)

                    Returns:
                        None

                    Raises:
                        SDKException:

                            if input data is not valid

                            if index server is cloud, not implemented error

                            if response is empty

                            if response is not success
        """
        if not isinstance(core_name, str):
            raise SDKException('IndexServers', '101')
        if self.is_cloud:
            raise SDKException('IndexServers', '104', "Not implemented for solr cloud")
        json_req = {"delete": {"query": self._create_solr_query(select_dict).replace("q=", "").replace("&wt=json", "")}}
        baseurl = f"{self.server_url[0]}/solr/{core_name}/update?commitWithin=1000&overwrite=true&wt=json"
        flag, response = self._cvpysdk_object.make_request("POST", baseurl, json_req)
        if flag and response.json():
            if 'error' in response.json():
                raise SDKException('IndexServers', '104', f' Failed with error message - '
                                                          f'{response.json().get("error").get("msg")}')
            if 'responseHeader' in response.json():
                commitstatus = str(response.json().get("responseHeader").get("status"))
                if int(commitstatus) != 0:
                    raise SDKException('IndexServers', '104',
                                       f"Deleting docs from the core returned bad status - {commitstatus}")
                return
        raise SDKException('IndexServers', '111')

    def hard_commit(self, core_name):
        """do hard commit for the given core name on index server

                    Args:

                        core_name               (str)  --  name of the solr core

                    Returns:
                        None

                    Raises:
                        SDKException:

                            if input data is not valid

                            if index server is cloud, not implemented error

                            if response is empty

                            if response is not success
        """
        if not isinstance(core_name, str):
            raise SDKException('IndexServers', '101')
        if self.is_cloud:
            raise SDKException('IndexServers', '104', "Not implemented for solr cloud")
        baseurl = f"{self.server_url[0]}/solr/{core_name}/update?commit=true"
        flag, response = self._cvpysdk_object.make_request("GET", baseurl)
        if flag and response.json():
            if 'error' in response.json():
                raise SDKException('IndexServers', '104', "Hard commit returned error")
            if 'responseHeader' in response.json():
                commitstatus = str(response.json()['responseHeader']['status'])
                if int(commitstatus) != 0:
                    raise SDKException('IndexServers', '104', "Hard commit returned bad status")
                return
        raise SDKException('IndexServers', '104', "Something went wrong with hard commit")

    
    def get_health_indicators(self, client_name=None):
        """Get health indicators for index server node by client name

                Args:
                    client_name     (str)       --  name of the client node

                Returns:
                    (response(str)) -- str json object

                Raises:

                    SDKException:
                        if input data is not valid
                        if client name is not passed for index server cloud
                        if response is not success
                        if response is empty

        """
        server_url = self.server_url[0]
        response = None
        if self.is_cloud or len(self.client_name) > 1:
            if client_name is None:
                raise SDKException('IndexServers', '104', 'Client name param missing for solr cloud')
            if client_name not in self.client_name:
                raise SDKException('IndexServers', '104', 'client name not found in this index server cloud')
            server_url = self.server_url[self.client_name.index(client_name)]
        baseurl = f"{server_url}/solr/rest/admin/healthsummary"
        headers = {
            'Accept': 'application/xml'
        }
        flag, response = self._cvpysdk_object.make_request("GET", headers=headers, url=baseurl)
        if flag:
            return response
        raise SDKException('IndexServers', '104', "Could not get health summary for [{}]".format(client_name))

    def get_all_cores(self, client_name=None):
        """gets all cores & core details from index server

                Args:
                    client_name     (str)       --  name of the client node
                        ***Applicable only for solr cloud mode or multi node Index Server***

                Returns:
                    (list,dict)     -- list containing core names
                                    -- dict containing details about cores

                Raises:

                    SDKException:

                        if input data is not valid

                        if client name is not passed for index server cloud

                        if response is not success

                        if response is empty

        """
        server_url = self.server_url[0]
        if self.is_cloud or len(self.client_name) > 1:
            if client_name is None:
                raise SDKException('IndexServers', '104', 'Client name param missing for solr cloud')
            if client_name not in self.client_name:
                raise SDKException('IndexServers', '104', 'client name not found in this index server cloud')
            server_url = self.server_url[self.client_name.index(client_name)]
        core_names = []
        baseurl = f"{server_url}/solr/admin/cores"
        flag, response = self._cvpysdk_object.make_request("GET", baseurl)
        if flag and response.json():
            if 'error' in response.json():
                raise SDKException('IndexServers', '104', "Unable to get core names from index server")
            if 'status' in response.json():
                for core in response.json()['status']:
                    core_names.append(core)
                return core_names, response.json()['status']
        raise SDKException('IndexServers', '104', "Something went wrong while getting core names")

    def _create_solr_query(self, select_dict=None, attr_list=None, op_params=None):
        """Method to create the solr query based on the params
            Args:
                select_dict     (dictionary)     --  Dictionary containing search criteria and value
                                                     Acts as 'q' field in solr query

                attr_list       (set)            --  Column names to be returned in results.
                                                     Acts as 'fl' in solr query

                op_params       (dictionary)     --  Other params and values for solr query
                                                        (Ex: start, rows)

            Returns:
                The solr url based on params

            Raises:
                SDKException:

                        if failed to form solr query
        """
        try:
            search_query = f'q='
            simple_search = 0
            if select_dict:
                for key, value in select_dict.items():
                    if isinstance(key, tuple):
                        if isinstance(value, list):
                            search_query += f'({key[0]}:{str(value[0])}'
                            for val in value[1:]:
                                search_query += f' OR {key[0]}:{str(val)}'
                        else:
                            search_query += f'({key[0]}:{value}'
                        for key_val in key[1:]:
                            if isinstance(value, list):
                                search_query += f' OR {key_val}:{str(value[0])}'
                                for val in value[1:]:
                                    search_query += f' OR {key_val}:{str(val)}'
                            else:
                                search_query += f' OR {key_val}:{value}'
                        search_query += ') AND '
                    elif isinstance(value, list):
                        search_query += f'({key}:{str(value[0])}'
                        for val in value[1:]:
                            search_query += f' OR {key}:{str(val)}'
                        search_query += ") AND "
                    elif key == "keyword":
                        search_query += "(" + value + ")"
                        simple_search = 1
                        break
                    else:
                        search_query = search_query + f'{key}:{str(value)} AND '
                if simple_search == 0:
                    search_query = search_query[:-5]
            else:
                search_query += "*:*"

            field_query = ""
            if attr_list:
                field_query = "&fl="
                for item in attr_list:
                    field_query += f'{str(item)},'
                field_query = field_query[:-1]
            if attr_list and 'content' in attr_list:
                field_query = f"{field_query}&exclude=false"

            ex_query = ""
            if not op_params:
                op_params = {'wt': "json"}
            else:
                op_params['wt'] = "json"
            for key, values in op_params.items():
                if isinstance(values, list):
                    for value in values:
                        ex_query += self.__form_field_query(key, value)
                else:
                    ex_query += self.__form_field_query(key, values)
            final_url = f'{search_query}{field_query}{ex_query}'
            return final_url
        except Exception as excp:
            raise SDKException('IndexServers', '104', f"Something went wrong while creating solr query - {excp}")

    def execute_solr_query(
            self,
            core_name,
            solr_client=None,
            select_dict=None,
            attr_list=None,
            op_params=None):
        """Creates solr url based on input and executes it on solr on given core/collection
            Args:

                core_name               (str)           --  Core name/collection name where we want to query

                solr_client             (str)           --  Index Server client name to execute solr query
                                                                Default : None (picks first client on index server)

                select_dict             (dictionary)    --  Dictionary containing search criteria and
                                                            value. Acts as 'q' field in solr query

                        Example :

                            1. General Criteria to filter results              -   {"jid": 1024, "datatype": 2,clid: 2}
                            2. Keyword Searches on solr                        -   {'keyword': 'SearchKeyword'}
                            3. For multiple value searches on single field     -   {'cvowner': ['xxx','yyy']}
                            4. For single value searches on multiple fields    -   {('cvowner','cvreaddisp') : 'xxx'}

                attr_list               (set)           --  Column names to be returned in results.
                                                                Acts as 'fl' in solr query

                        Example (For Exchange Mailbox IDA, below fields are there in solr) :
                                    {
                                     'msgclass',
                                     'ccsmtp',
                                     'fmsmtp',
                                     'folder'
                                   }

                op_params               (dictionary)    --  Other params and values for solr query. Do not
                                                            mention 'wt' param as it is always json

                                                            Example : {"rows": 0}

            Returns:
                content of the response

            Raises:
                SDKException:

                        if unable to send request

                        if response is not success
        """
        solr_url = f"solr/{core_name}/select?{self._create_solr_query(select_dict, attr_list, op_params)}"
        if solr_client is None:
            solr_url = f"{self.server_url[0]}/{solr_url}"
        else:
            if solr_client not in self.client_name:
                raise SDKException('IndexServers', '104', 'client name not found in this index server')
            server_url = self.server_url[self.client_name.index(solr_client)]
            solr_url = f"{server_url}/{solr_url}"
        flag, response = self._cvpysdk_object.make_request("GET", solr_url)
        if flag and response.json():
            return response.json()
        elif response.status_code == httplib.FORBIDDEN:
            cmd = f"(Invoke-WebRequest -UseBasicParsing -uri \"{solr_url}\").content"
            client_obj = None
            if solr_client:
                client_obj = self._commcell_obj.clients.get(solr_client)
            else:
                # if no client is passed, then take first client in index server cloud
                client_obj = self._commcell_obj.clients.get(self.client_name[0])
            exit_code, output, error_message = client_obj.execute_script(script_type="PowerShell",
                                                                         script=cmd)
            if exit_code != 0:
                raise SDKException(
                    'IndexServers',
                    '104',
                    f"Something went wrong while querying solr - {exit_code}")
            elif error_message:
                raise SDKException(
                    'IndexServers',
                    '104',
                    f"Something went wrong while querying solr - {error_message}")
            try:
                return json.loads(output.strip())
            except Exception:
                raise SDKException('IndexServers', '104', f"Something went wrong while querying solr - {output}")
        raise SDKException('IndexServers', '104', "Something went wrong while querying solr")

    def get_index_node(self, node_name):
        """Returns an Index server node object for given node name
            Args:
                node_name           (str)   --  Index server node name

            Returns:
                IndexNode class object

            Raises:
                SDKException:

                        if node not found for the given node name

        """
        node_name = node_name.lower()
        if node_name in self.client_name:
            return IndexNode(self._commcell_obj, self.engine_name, node_name)
        raise SDKException("IndexServers", '104', 'Index server node not found')

    def get_plan_info(self):
        """Gets the plan information of the index server
            Returns:
                dict - containing the plan information
        """
        client = self._commcell_obj.clients.get(self.index_server_client_id)
        instance_props = client.properties.get("pseudoClientInfo", {}).get("distributedClusterInstanceProperties", {})
        plan_details = instance_props.get("clusterConfig",{}).get("cloudInfo", {}).get("planInfo", {})
        return plan_details

    def get_os_info(self):
        """Returns the OS type for the Index server"""

        nodes_name = self.client_name
        nodes = [self._commcell_obj.clients.get(node) for node in nodes_name]
        nodes_os_info = [node.os_info for node in nodes]
        if IndexServerOSType.WINDOWS.value.lower() in nodes_os_info[0].lower():
            for node in nodes_os_info[1:]:
                if IndexServerOSType.UNIX.value.lower() in node.lower():
                    return IndexServerOSType.MIXED.value
            return IndexServerOSType.WINDOWS.value
        else:
            for node in nodes_os_info[1:]:
                if IndexServerOSType.WINDOWS.value.lower() in node.lower():
                    return IndexServerOSType.MIXED.value
            return IndexServerOSType.UNIX.value

    def __form_field_query(self, key, value):
        """
        Returns the query with the key and value passed
        Args:
                key(str)    -- key for forming the query
                value(str)  -- value for forming the query
            Returns:
                query to be executed against solr
        """
        query = None
        if value is None:
            query = f'&{key}'
        else:
            query = f'&{key}={str(value)}'
        return query

    @property
    def plan_name(self):
        """Returns the plan name associated with index server
            Returns:
                str - name of the plan
        """
        return self.plan_info.get("planName")

    @property
    def os_info(self):
        """Returns the OS type for the Index server"""
        return self.os_type

    @property
    def is_cloud(self):
        """Returns true if the Index server is cloud and false if not"""
        return self.server_type == 5

    @property
    def nodes_count(self):
        """Returns the count of Index server nodes"""
        return len(self.client_id)

    @property
    def roles_data(self):
        """Returns the cloud roles data"""
        return self._roles_obj.roles_data

    @property
    def properties(self):
        """Returns the index server properties"""
        return self._properties

    @property
    def host_name(self):
        """Returns a list of host names of all index server nodes"""
        return self._properties[IndexServerConstants.HOST_NAME]

    @property
    def cloud_name(self):
        """Returns the internal cloud name of index server"""
        return self._properties[IndexServerConstants.CLOUD_NAME]

    @property
    def client_name(self):
        """Returns a list of client names of all index server nodes"""
        return self._properties[IndexServerConstants.CLIENT_NAME]

    @property
    def server_url(self):
        """Returns a list of Solr url of all index server nodes"""
        return self._properties[IndexServerConstants.CI_SERVER_URL]

    @property
    def type(self):
        """Returns the type of index server"""
        return self._properties[IndexServerConstants.TYPE]

    @property
    def base_port(self):
        """Returns a list of base ports of all index server nodes"""
        return self._properties[IndexServerConstants.BASE_PORT]

    @property
    def client_id(self):
        """Returns a list client ids of all index server nodes"""
        return self._properties[IndexServerConstants.CLIENT_ID]

    @property
    def roles(self):
        """Returns a list of roles of index server"""
        return self._properties[IndexServerConstants.ROLES]

    @property
    def role_display_name(self):
        """Returns the roles display name of index server"""
        role_disp_name = []
        for role_version in self.roles:
            for role in self.roles_data:
                if role_version == role['roleVersion']:
                    role_disp_name.append(role['roleName'])
                    break
        return role_disp_name

    @property
    def cloud_id(self):
        """Returns the cloud id of index server"""
        return self._properties[IndexServerConstants.CLOUD_ID]

    @property
    def server_type(self):
        """Returns the server type of index server"""
        return self._properties[IndexServerConstants.SERVER_TYPE]

    @property
    def engine_name(self):
        """Returns the engine name of index server"""
        return self._properties[IndexServerConstants.ENGINE_NAME]

    @property
    def index_server_client_id(self):
        """Returns the index server client id of index server"""
        return self._properties[IndexServerConstants.INDEX_SERVER_CLIENT_ID]

    @property
    def fs_collection(self):
        """Returns the multinode collection name of File System Index

            Returns:

                str --  File System index multinode collection name

        """
        return f'fsindex_{"".join(letter for letter in self.cloud_name if letter.isalnum())}_multinode'


class IndexNode(object):
    """Class for Index server node object"""

    def __init__(self, commcell_obj, index_server_name, node_name):
        """Initialize the IndexNode class

            Args:
                commcell_obj        (object)    --  commcell object
                index_server_name   (int)       --  Index server name
                node_name           (str)       --  Index server node client name

        """
        self.commcell = commcell_obj
        self.index_server_name = index_server_name
        self.data_index = 0
        self.index_server = None
        self.index_node_name = node_name.lower()
        self.index_node_client = None
        self.index_client_properties = None
        self.refresh()

    def refresh(self):
        """Refresh the index node properties"""
        self.commcell.index_servers.refresh()
        self.index_server = self.commcell.index_servers.get(self.index_server_name)
        self.data_index = self.index_server.client_name.index(self.index_node_name)
        self.commcell.clients.refresh()
        self.index_node_client = self.commcell.clients.get(self.index_node_name)
        self.index_client_properties = self.index_node_client.properties['pseudoClientInfo']['indexServerProperties']

    @property
    def node_name(self):
        """Returns Index server node client name"""
        return self.index_server.client_name[self.data_index]

    @property
    def node_id(self):
        """Returns Index server node client id"""
        return self.index_server.client_id[self.data_index]

    @property
    def solr_port(self):
        """Returns port number Solr is running on the Index server node"""
        return self.index_server.base_port[self.data_index]

    @property
    def solr_url(self):
        """Returns Solr URL for Index server node"""
        return self.index_server.server_url[self.data_index]

    @property
    def roles(self):
        """Returns the array of roles installed with the index server within the commcell"""
        return self.index_server.role_display_name

    @property
    def index_location(self):
        """Returns Index directory for the Index server Node"""
        node_meta_infos = self.index_client_properties['nodeMetaInfos']
        for info in node_meta_infos:
            if info['name'] == 'INDEXLOCATION':
                return info['value']
        return None

    @property
    def jvm_memory(self):
        """Returns Solr JVM memory for the Index server Node"""
        node_meta_infos = self.index_client_properties['nodeMetaInfos']
        for info in node_meta_infos:
            if info['name'] == 'JVMMAXMEMORY':
                return info['value']
        return None

    @solr_port.setter
    def solr_port(self, port_no):
        """Setter to set the Solr port number for the node

            Args:
                port_no     (str)   --  Solr port number to be set for node

        """
        solr_port_param = deepcopy(IndexServerConstants.SOLR_PORT_META_INFO)
        solr_port_param['value'] = str(port_no)
        cloud_param = [solr_port_param]
        self.index_server.modify(self.index_location, self.index_node_name, cloud_param)
        self.refresh()

    @jvm_memory.setter
    def jvm_memory(self, memory):
        """Setter to set the Solr JVM memory for the node

                    Args:
                        memory      (str)   --  Solr JVM memory to be set for the node

        """
        solr_jvm_param = deepcopy(IndexServerConstants.SOLR_JVM_META_INFO)
        solr_jvm_param['value'] = str(memory)
        solr_port_param = deepcopy(IndexServerConstants.SOLR_PORT_META_INFO)
        solr_port_param['value'] = str(self.solr_port)
        cloud_param = [solr_jvm_param, solr_port_param]
        self.index_server.modify(self.index_location, self.index_node_name, cloud_param)
        self.refresh()


class _Roles(object):
    """Class for cloud roles data operations"""

    def __init__(self, commcell_object):
        """Initializes _Roles class with commcell object

            Args:
                commcell_object (object)    --  instance of Commcell class

            Returns:
                object  -   instance of _Roles class
        """
        self.commcell_object = commcell_object
        self._roles_data = None
        self.refresh()

    def refresh(self):
        """Refreshes the class data"""
        self._get_all_roles()

    def _get_all_roles(self):
        """Method to get all cloud roles details available on the commcell.

            Raises:
                SDKException:
                    Response was empty.

                    Response was not success.
        """
        flag, response = self.commcell_object._cvpysdk_object.make_request(
            "GET", self.commcell_object._services['GET_ALL_ROLES']
        )
        if flag:
            if response.json():
                if 'rolesInfo' in response.json():
                    self._roles_data = response.json()['rolesInfo']
                    return
            raise SDKException('Response', '102')
        raise SDKException('Response', '101')

    def get_role_id(self, role_name):
        """Method to get a cloud role id with given name

            Args:
                role_name       (str)   --  cloud role name of which role id has to be returned

            Returns:
                role_id         (int)   --  if role name is found in roles data then returns the id
                                            else returns None

        """
        for role_data in self._roles_data:
            if role_data['roleName'] == role_name:
                return role_data['roleId']
        return None

    def update_roles_data(self):
        """Synchronize the cloud role data with the commcell database"""
        self._get_all_roles()

    @property
    def roles_data(self):
        """Returns the list of dictionary of details of each cloud role"""
        return self._roles_data



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------------
# Copyright Commvault Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# --------------------------------------------------------------------------
from cvpysdk.exception import SDKException
from cvpysdk.instances.vsinstance import VirtualServerInstance


class NullSubclient(VirtualServerInstance):

    def __init__(self, agent_object, instance_name, instance_id=None):
        raise SDKException('Instance', '102',
                           'Instance: "{0}" is not yet supported'.
                           format(instance_name))




# -*- coding: utf-8 -*-

# --------------------------------------------------------------------------
# Copyright Commvault Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# --------------------------------------------------------------------------

"""Utilities class for Activate application : Compliance Search

ComplianceSearchUtils:  Class for compliance search related operations support

ExportSets: Class for representing all the export sets associated with the commcell

ExportSet:  Class for an instance of a single Export set of the commcell

Export: Class for an instance of a single Export of the commcell


ComplianceSearchUtils
============

    __init__()                          --  Initializes the Compliance search utility class

    _response_not_success()             --  Helper method to raise exception when response is not 200 (ok)

    do_compliance_search()              --  Method to run a compliance search with the search text provided


ExportSets
============

    __init__()                          --  Initializes the ExportSets class

    _response_not_success()             --  Helper method to raise exception when response is not 200 (ok)

    _get_export_sets()                  --  Method to call the API and fetch all the export sets
                                            from the commcell environment

    refresh()                           --  Method to refresh all the properties of the class ExportSets

    add()                               --  Method to create an export set to the commcell environment

    has()                               --  Method to check if the export set exists or not

    get()                               --  Method to get the export set

    delete()                            --  Method to delete the export set

ExportSet
============

    __init__()                          --  Initializes the ExportSet class

    _response_not_success()             --  Helper method to raise exception when response is not 200 (ok)

    refresh()                           --  Method to refresh all the properties of the class ExportSet

    _get_all_exports()                  --  Method to fetch all the exports for the export set

    share()                             --  Method to share the export set with the user or user group

    has()                               --  Method to check if the export exists or not

    get()                               --   Method to get the export

    delete()                            --  Method to delete the export

    export_items_to_set()               --  Method to export items/documents to the export set

    select()                            --  Static Method to randomly pick user input
                                            amount of items from the search result items

ExportSet Attributes
-----------------------

    **properties**                      --  return all the properties of the export set

    **export_set_full_name**            --  return the export set full name

    **export_set_name**                 --  return the export set name

    **export_set_comment**              --  return the export set comment

    **export_set_id**                   --  return the export set ID

    **export_set_guid**                 --  return the export set GUID

    **export_set_owner_info**           --  return the export set owner info


Export
============

    __init__()                          --  Initializes the Export class

    _response_not_success()             --  Helper method to raise exception when response is not 200 (ok)

    refresh()                           --  Method to refresh all the properties of the class Export

    download_export()                   --  Method to download the exported items to a zip file

"""
import copy
import base64
import os.path
import random
import requests

from cvpysdk.exception import SDKException
from cvpysdk.activateapps.constants import ComplianceConstants


class ComplianceSearchUtils():
    """Class for compliance search related operations support"""

    def __init__(self, commcell):
        """Initializes the Compliance search utility class"""
        self._commcell = commcell
        self._index_servers = commcell.index_servers
        self._cvpysdk_object = commcell._cvpysdk_object
        self._services = commcell._services
        self._update_response_ = commcell._update_response_
        self._users = commcell.users
        self._export_sets = ExportSets(commcell)
        self._do_search_api = self._services["DO_COMPLIANCE_SEARCH"]

    def _response_not_success(self, response):
        """Helper method to raise exception when response is not 200 (ok)

            Raises:
                SDKException:
                    Response was not success
        """
        raise SDKException(
            'Response',
            '101',
            self._update_response_(
                response.text))

    def do_compliance_search(self, search_text, index_server_name=None,
                             page_size=50, app_type=ComplianceConstants.AppTypes.FILE_SYSTEM):
        """Method to run a compliance search with the search text provided

            Args:
                search_text         (str)   -   Search text to be searched on the Compliance search
                index_server_name   (str)   -   Index server name on which the search has to be executed
                page_size           (int)   -   Search result page size value (To Fetch all HITS - 0)
                                                Default: 50
                app_type            (str)   -   ComplianceConstants.AppTypes Enum values
                                                Default: FILE_SYSTEM

            Returns:
                (list)  -   List of all the search result items with the metadata and SOLR fields
                Example :
                    [ {
                        "FileName": <name>,
                        "SizeKB": <size>...,
                        <name>: <key>
                    },
                    {
                        "FileName": <name>,
                        "SizeKB": <size>...,
                        <name>: <key>
                    }... ]

            Raises:
                    SDKException:
                        Response was not success

        """
        self._index_servers.refresh()
        search_request = copy.deepcopy(ComplianceConstants.COMPLIANCE_SEARCH_JSON)
        if index_server_name:
            if not self._index_servers.has(index_server_name):
                raise SDKException('IndexServers', '102')
            search_request["listOfCIServer"][0]["cloudID"] = self._index_servers.get(index_server_name).cloud_id
        else:
            del search_request["listOfCIServer"]
        search_request["advSearchGrp"]["cvSearchKeyword"]["keyword"] = search_text
        search_request["userInformation"]["userGuid"] = self._users.get(self._commcell.commcell_username).user_guid
        if app_type in ComplianceConstants.FILE_TYPES:
            search_request["advSearchGrp"]["galaxyFilter"][0]["applicationType"] = ComplianceConstants.FILE_TYPE
            search_request["advSearchGrp"][ComplianceConstants.FILE_FILTERS_KEY] = ComplianceConstants.FILE_FILTERS
            custom_facet = copy.deepcopy(ComplianceConstants.FILE_FACET)
            if app_type != ComplianceConstants.AppTypes.FILE_SYSTEM:
                custom_facet = copy.deepcopy(ComplianceConstants.CUSTOM_FACET)
                custom_facet[0]["name"] = ComplianceConstants.CUSTOM_FACETS_NAME[app_type]
                if app_type == ComplianceConstants.AppTypes.TEAMS:
                    search_request["searchProcessingInfo"]["queryParams"].append(
                        {
                            "param": "RESPONSE_FIELD_LIST",
                            "value": ComplianceConstants.RESPONSE_FIELD_LIST
                        },
                    )
                custom_facet[1]["stringParameter"][0]["name"] = ComplianceConstants.CUSTOM_FACETS[app_type]
            search_request["facetRequests"][ComplianceConstants.FACET_KEY] = custom_facet
        elif app_type in ComplianceConstants.EMAIL_TYPES:
            search_request["advSearchGrp"]["galaxyFilter"][0]["applicationType"] = ComplianceConstants.EMAIL_TYPE
            search_request["advSearchGrp"][ComplianceConstants.EMAIL_FILTERS_KEY] = {
                "usermailbox": app_type == ComplianceConstants.AppTypes.EXCHANGE,
                "journalmailbox": app_type == ComplianceConstants.AppTypes.EXCHANGE_JOURNAL,
                "smtpmailbox": app_type == ComplianceConstants.AppTypes.EXCHANGE_JOURNAL
            }
        else:
            raise SDKException('ComplianceSearch', '107')
        if page_size == 0:
            flag, response = self._cvpysdk_object.make_request('POST', self._do_search_api, payload=search_request)
            if flag and response.json() and "totalHits" in response.json().get("proccessingInfo"):
                page_size = max(response.json()["proccessingInfo"]["totalHits"], 50)
        search_request["searchProcessingInfo"]["pageSize"] = page_size
        flag, response = self._cvpysdk_object.make_request('POST', self._do_search_api, payload=search_request)
        if flag:
            if response.json():
                if "searchResult" in response.json():
                    if "resultItem" in response.json()["searchResult"]:
                        return response.json()["searchResult"]["resultItem"]
                return []
            raise SDKException('Response', '102')
        else:
            self._response_not_success(response)


class ExportSets():
    """Class for representing all the export sets associated with the commcell"""

    def __init__(self, commcell):
        """Initializes the ExportSets class"""
        self._commcell = commcell
        self._cvpysdk_object = None
        self._services = None
        self._update_response_ = None
        self._user_guid = None
        self._add_export_set_api = None
        self._add_export_set_json_req = None
        self._get_export_sets_api = None
        self._get_export_sets_json_req = None
        self._delete_export_set_api = None
        self._delete_export_set_json_req = None
        self._all_export_set = None
        self.refresh()

    def _get_export_sets(self):
        """Method to call the API and fetch all the export sets from the commcell environment"""
        flag, response = self._cvpysdk_object.make_request(
            method="POST", url=self._get_export_sets_api, payload=self._get_export_sets_json_req)
        if flag:
            if response.json() and "containers" in response.json():
                containers = response.json()['containers']
                for container in containers:
                    self._all_export_set.update(
                        {container["containerName"]: container})
        else:
            self._response_not_success(response)

    def refresh(self):
        """Method to refresh all the properties of the class ExportSets"""
        self._cvpysdk_object = self._commcell._cvpysdk_object
        self._services = self._commcell._services
        self._update_response_ = self._commcell._update_response_
        self._user_guid = self._commcell.users.get(self._commcell.commcell_username).user_guid
        self._all_export_set = {}
        self._get_export_sets_api = self._services['GET_EXPORT_SETS']
        self._get_export_sets_json_req = \
            {
                "getContainerOptions": 0,
                "entityType": 9503,
                "userGuid": self._user_guid,
                "attribute": {
                    "all": True
                }
            }
        self._add_export_set_api = self._services['ADD_EXPORT_SET']
        self._add_export_set_json_req = \
            {
                "entityType": 9503,
                "operationType": 1,
                "userGuid": self._user_guid,
                "fromSite": 2,
                "container": {
                    "containerType": 9503,
                    "containerName": None,
                    "containerOwnerType": 1,
                    "comment": "Export set created from CvPySDK"
                }
            }
        self._delete_export_set_api = self._services['DELETE_EXPORT_SET']
        self._delete_export_set_json_req = \
            {
                "entityType": 9503,
                "containers": None
            }
        self._get_export_sets()

    def _response_not_success(self, response):
        """Helper method to raise exception when response is not 200 (ok)

            Raises:
                SDKException:
                    Response was not success
        """
        raise SDKException(
            'Response',
            '101',
            self._update_response_(
                response.text))

    def add(self, export_set_name, comment=None):
        """Method to create an export set to the commcell environment

            Args:
                export_set_name     (str)   -   Export set name for the newly created export set
                comment             (str)   -   Export set description fot the newly created export set

            Returns:
                ExportSet instance for the newly created export set

        """
        if not self.has(export_set_name):
            add_export_set_json_req = copy.deepcopy(self._add_export_set_json_req)
            add_export_set_json_req['container']['containerName'] = export_set_name
            if comment:
                add_export_set_json_req['container']['comment'] = comment
            flag, response = self._cvpysdk_object.make_request(
                method="POST", url=self._add_export_set_api, payload=add_export_set_json_req)
            if flag:
                if response.json() and "container" in response.json():
                    self.refresh()
                    return self.get(export_set_name=export_set_name)
                raise SDKException('Response', '102')
            else:
                self._response_not_success(response)

    def has(self, export_set_name):
        """Method to check if the export set exists or not

            Args:
                export_set_name (str)   -   Export set name to be checked

            Returns:
                Returns True if export set exists in the environment or False otherwise

        """
        return export_set_name in self._all_export_set

    def get(self, export_set_name):
        """Method to get the export set

            Args:
                export_set_name (str)   -   Export set name to get

            Returns:
                ExportSet instance for the export set with the given name if found else raises Exception

        """

        if self.has(export_set_name):
            return ExportSet(self._commcell, self._all_export_set[export_set_name])
        raise SDKException('ComplianceSearch', '106')

    def delete(self, export_set_name):
        """Method to delete the export set

            Args:
                export_set_name (str)   -   Export set name to be deleted

            Returns:
                Returns True if delete successfully else raises error

            Raises:
                SDKException:
                    Response was not success

        """
        if not self.has(export_set_name):
            raise SDKException("ComplianceSearch", "105")
        delete_json = copy.deepcopy(self._delete_export_set_json_req)
        delete_json['containers'] = [self._all_export_set[export_set_name]]
        flag, response = self._cvpysdk_object.make_request("POST", self._delete_export_set_api, delete_json)
        if flag:
            if response.json() and "errList" in response.json() and len(response.json()["errList"]) == 0:
                self.refresh()
                return True
        self._response_not_success(response)


class ExportSet():
    """Class for an instance of a single Export set of the commcell"""

    def __init__(self, commcell, export_set_properties):
        """Initializes the ExportSet class"""
        self._commcell = commcell
        self._export_set_properties = export_set_properties
        self._cvpysdk_object = None
        self._services = None
        self._update_response_ = None
        self._user_guid = None
        self._get_exports_api = None
        self._get_export_json_req = None
        self._export_items_api = None
        self._export_items_json_req = None
        self._share_export_set_api = None
        self._exports_list = None
        self._all_exports = None
        self._delete_export_api = None
        self._delete_export_json_req = None
        self.refresh()

    def refresh(self):
        """Method to refresh the properties of the class ExportSet"""
        self._cvpysdk_object = self._commcell._cvpysdk_object
        self._services = self._commcell._services
        self._update_response_ = self._commcell._update_response_
        self._user_guid = self._commcell.users.get(self._commcell.commcell_username).user_guid
        self._get_exports_api = self._services['GET_EXPORTS']
        self._share_export_set_api = self._services['SECURITY_ASSOCIATION']
        self._get_export_json_req = \
            {
                "containerOpReq": {
                    "entityType": 9503,
                    "operationType": 1,
                    "userGuid": self._user_guid,
                    "fromSite": 0,
                    "container": self._export_set_properties
                },
                "filter": {},
                "pagingData": {
                    "startIndex": 0,
                    "fetchEmails": True,
                    "pageSize": 50,
                    "fetchFiles": True,
                    "orderByClause": "CreateTime desc"
                }
            }
        self._export_items_api = self._services['EXPORT_ITEM_TO_SET']
        self._export_items_json_req = \
            {
                "complianceData": {
                    "mode": 2,
                    "restoreType": 2,
                    "downLoadDesc": None,
                    "destContainer": None,
                    "originatingContainer": {
                        "containerOwnerType": 1
                    },
                    "options": {
                        "zipEML": True,
                        "retentionInDays": 30}
                },
                "onlineData": {
                    "downloadStatus": 0
                },
                "listOfItems": {
                    "resultItem": None
                }
            }
        self._delete_export_api = self._services['GET_EXPORTS']
        self._delete_export_json_req = \
            {
                "operationType": 2,
                "container": None,
                "downloadItems": {
                    "items": None
                },
                "tagModifyRequest": {
                    "operationType": 2
                }
            }
        self._all_exports = {}
        self._get_all_exports()

    def _response_not_success(self, response):
        """Helper method to raise exception when response is not 200 (ok)

        Raises:
            SDKException:
                Response was not success

        """
        raise SDKException(
            'Response',
            '101',
            self._update_response_(
                response.text))

    def _get_all_exports(self):
        """Method to fetch all the exports for the export set"""
        flag, response = self._cvpysdk_object.make_request(
            method="POST", url=self._get_exports_api, payload=self._get_export_json_req)
        if flag:
            if (response.json() and "totalHits" in response.json()
                    and int(response.json()['totalHits']) != 0):
                items = response.json()['downloadItems']['items']
                for item in items:
                    self._all_exports.update(
                        {item["description"]: item})
            else:
                self._all_exports = {}
        else:
            self._response_not_success(response)

    def share(self, user_or_user_group_name=None, permissions=None, mode=2):
        """Method to share the export set with the user or user group provided

            Args:
                user_or_user_group_name (str)       -   User or user group with which export set has to be shared
                permissions             (list/str)  -   List or comma separated permissions that need
                                                        to be set to the user/ user group
                mode                    (int)       -   to add (2), remove (3) or overwrite (1) the permissions
                                                        Default : Add (2)

            Returns:
                Returns True if share worked fine else raises an Exception

        """
        if isinstance(permissions, str):
            permissions = permissions.split(",")
        share_json = copy.deepcopy(ComplianceConstants.EXPORT_SET_SHARE_REQUEST_JSON)
        unshare = False
        user_details = {}
        if user_or_user_group_name is None:
            unshare = True
        elif self._commcell.users.has_user(user_or_user_group_name):
            user_details = {
                "userId": int(self._commcell.users.get(user_or_user_group_name).user_id),
                "userName": user_or_user_group_name,
                "_type_": 13
            }
        elif self._commcell.user_groups.has_user_group(user_or_user_group_name):
            user_details = {
                "groupId": int(self._commcell.user_groups.get(user_or_user_group_name).user_group_id),
                "userGroupName": user_or_user_group_name,
                "_type_": 15
            }
        else:
            raise SDKException('ComplianceSearch', '101')
        if unshare:
            share_json['securityAssociations']['associations'] = []
            share_json['securityAssociations']['associationsOperationType'] = 1
        else:
            share_json['securityAssociations']['associationsOperationType'] = mode
            share_json['securityAssociations']['associations'][0]['userOrGroup'] = [user_details]
            if permissions is None:
                share_json['securityAssociations']['associations'][0]['properties']['permissions'] = []
                permissions = [["View", "Download"], list(ComplianceConstants.PERMISSIONS.keys())][mode//3]
            if mode == 3:
                share_json['securityAssociations']['associations'][0]['properties']['permissions'] = []
            for permission in permissions:
                try:
                    share_json['securityAssociations']['associations'][0]['properties']['permissions'].append(
                        ComplianceConstants.PERMISSIONS[permission]
                    )
                except KeyError:
                    raise SDKException('ComplianceSearch', '102')
        share_json['entityAssociated']['entity'][0]['downloadSetId'] = self.export_set_id
        flag, response = self._cvpysdk_object.make_request("POST", self._share_export_set_api, share_json)
        if flag:
            if response.json() and "response" in response.json():
                if response.json()['response'][0]['errorCode'] == 0:
                    self.refresh()
                    return True
        self._response_not_success(response)

    def has(self, export_name):
        """Method to check if the export exists or not

            Args:
                export_name (str)   -   Export name to be checked

            Returns:
                Returns True if export exists in the environment or False otherwise

        """
        return export_name in self._all_exports

    def get(self, export_name):
        """Method to get the export

            Args:
                export_name (str)   -   Export name to get

            Returns:
                Export class instance for the export with the given name if found else returns None

        """
        if self.has(export_name):
            return Export(self._commcell, self._all_exports[export_name])
        raise SDKException('ComplianceSearch', '103')

    def delete(self, export_name):
        """Method to delete the export

            Args:
                export_name (str)   -   Export name to be deleted

            Returns:
                Returns True if delete successfully else raises error

            Raises:
                SDKException:
                    Response was not success

        """
        if not self.has(export_name):
            raise SDKException("ComplianceSearch", "103")
        delete_json = copy.deepcopy(self._delete_export_json_req)
        delete_json["downloadItems"]["items"] = [self._all_exports[export_name]]
        delete_json["container"] = self.properties
        flag, response = self._cvpysdk_object.make_request("POST", self._delete_export_api, delete_json)
        if flag:
            if response.json() and "errList" in response.json() and len(response.json()["errList"]) != 0:
                if response.json()["errList"][0]["errorCode"] != 0:
                    raise SDKException("ComplianceSearch", "104",
                                       response.json()["errList"][0].get("errLogMessage"))
            return True
        self._response_not_success(response)

    def export_items_to_set(self, export_name, export_items, export_type=ComplianceConstants.ExportTypes.CAB):
        """Method to export items/documents to the export set

            Args:
                export_name     (str)   -   Export name for the exported items
                export_items    (list)  -   List of search result items which needs to be export
                export_type     (str)   -   ComplianceConstants.ExportTypes Enum values
                                            Default: CAB

            Returns:
                Returns the restore job ID for the export operation

            Raises:
                SDKException:
                    Response was not success

        """
        if export_type not in ComplianceConstants.RESTORE_TYPE:
            raise SDKException('ComplianceSearch', '108')
        export_items_json_req = copy.deepcopy(self._export_items_json_req)
        export_items_json_req['complianceData']['restoreType'] = ComplianceConstants.RESTORE_TYPE[export_type]
        export_items_json_req['complianceData']['downLoadDesc'] = export_name
        export_items_json_req['complianceData']['destContainer'] = self.properties
        export_items_json_req['listOfItems']['resultItem'] = export_items
        flag, response = self._cvpysdk_object.make_request(
            method="POST", url=self._export_items_api, payload=export_items_json_req)
        if flag:
            if response.json() and "downloadId" in response.json() and "jobId" in response.json():
                self.refresh()
                return response.json()['jobId']
            raise SDKException('Response', '102')
        else:
            self._response_not_success(response)

    @staticmethod
    def select(result_items, no_of_files=0, export_all=False):
        """Static Method to randomly pick user input amount of items from the search result items

            Args:
                result_items    (list)      -   List of all the search result items
                no_of_files     (int)       -   Number of items to be selected
                export_all      (bool)      -   if all the items needs to be selected or not

            Returns:
                List of randomly selected (no_of_files) items from the (result_items)

        """
        if export_all or len(result_items) < no_of_files:
            return result_items
        return list(random.sample(result_items, no_of_files))

    @property
    def properties(self):
        """Method to return all the properties of the export set

            Returns:
                Dict of the properties of the export set as received from the API (rawdata)

        """
        return self._export_set_properties

    @property
    def export_set_full_name(self):
        """Method to return the export set full name

            Returns:
                (str) export set full name

        """
        return self._export_set_properties['containerFullName']

    @property
    def export_set_name(self):
        """Method to return the export set name

            Returns:
                (str) export set name

        """
        return self._export_set_properties['containerName']

    @property
    def export_set_comment(self):
        """Method to return the export set comment

            Returns:
                (str) export set comment

        """
        return self._export_set_properties['comment']

    @property
    def export_set_id(self):
        """Method to return the export set ID

            Returns:
                (int) export set ID

        """
        return self._export_set_properties['containerId']

    @property
    def export_set_guid(self):
        """Method to return the export set Guid

            Returns:
                (str) export set Guid

        """
        return self._export_set_properties['containerGuid']

    @property
    def export_set_owner_info(self):
        """Method to return the export set Owner info

            Returns:
                (str) export set Owner info

        """
        return self._export_set_properties['ownerInfo']


class Export():
    """Class for an instance of a single Export of the export set"""

    def __init__(self, commcell, export_properties):
        """Initializes the Export class instance"""
        self._commcell = commcell
        self._export_set_properties = export_properties
        self._cvpysdk_object = None
        self._services = None
        self._update_response_ = None
        self._user_guid = None
        self._download_file_api = None
        self._download_file_json_req = None
        self.refresh()

    def _response_not_success(self, response):
        """Helper method to raise exception when response is not 200 (ok)

        Raises:
            SDKException:
                Response was not success

        """
        raise SDKException(
            'Response',
            '101',
            self._update_response_(
                response.text))

    def refresh(self):
        """Method to refresh the properties of the class Export"""
        self._cvpysdk_object = self._commcell._cvpysdk_object
        self._services = self._commcell._services
        self._update_response_ = self._commcell._update_response_
        self._user_guid = self._commcell.users.get(self._commcell.commcell_username).user_guid
        self._download_file_api = self._services['DOWNLOAD_EXPORT_ITEMS']
        self._download_file_json_req = \
            {
                "appTypeId": 200,
                "responseFileName": self._export_set_properties['description'],
                "fileParams": [
                    {
                        "name": self._export_set_properties['downLoadID'],
                        "id": 2
                    },
                    {
                        "name": "zip",
                        "id": 3
                    }
                ]
            }

    def download_export(self, download_folder):
        """Method to download the exported items to a zip file

            Args:
                download_folder     (str)   -   Path of the folder in which exported items zip file should be saved

            Returns:
                (str) path of the downloaded zip file

            Raises:
                SDKException:
                    Response was not success

        """
        flag, response = self._cvpysdk_object.make_request(
            method="POST", url=self._download_file_api, payload=self._download_file_json_req)
        if flag:
            if response.json() and "fileContent" in response.json():
                file_content = response.json()['fileContent']
                download_file = os.path.join(download_folder, file_content['fileName'])
                if 'cloudUrl' in response.json():
                    url = response.json()['cloudUrl']
                    cloud_file = requests.get(url)
                    if cloud_file.status_code == 200:
                        file_data = cloud_file.content
                    else:
                        raise SDKException('ComplianceSearch', '109')
                else:
                    file_data = base64.b64decode(file_content['data'])
                with open(download_file, "wb") as f:
                    f.write(file_data)
                return download_file
            raise SDKException('Response', '102')
        else:
            self._response_not_success(response)



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------------
# Copyright Commvault Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# --------------------------------------------------------------------------

"""
File for operating on a MYSQL Instance.

MYSQLInstance is the only class defined in this file.

MYSQLInstance: Derived class from Instance Base class, representing an
                MYSQL instance, and to perform operations on that instance

MYSQLInstance:
==============

    _get_instance_properties()      -- method to get the properties of the instance

    _restore_json()                 -- returns the apppropriate JSON request to pass for
    Restore In-Place

    restore_in_place()              -- Restores the mysql data/log files specified in the
    input paths list to the same location

    restore_out_of_place()          -- method to perform out of place restore of MySQL data/log/recurring files to the
    destination client.

    _restore_browse_option_json()   -- setter for  browse option  property in restore

    _restore_common_options_json()  -- setter for common options property in restore

    _restore_destination_json()     -- setter for destination options property in restore

    _restore_fileoption_json()      -- setter for file option property in restore

    _restore_admin_option_json()    -- setter for admin option property in restore

    _restore_mysql_option_json()    -- setter for MySQL restore option property in restore


MYSQLInstance instance Attributes:
==================================

    **port**                            -- Returns the MySQL Server Port number

    **mysql_username**                  -- Returns the MySQL Server username

    **nt_username**                     -- Returns the MySQL Server nt username

    **config_file**                     -- Returns the MySQL Server Config File location

    **binary_directory**                -- Returns the MySQL Server Binary File location

    **version**                         -- Returns the MySQL Server version number

    **log_data_directory**              -- Returns the MySQL Server log data directory

    **log_backup_sp_details**           -- Returns the MySQL Server Log backup SP details

    **command_line_sp_details**         -- Returns the MySQL Server commandline SP details

    **autodiscovery_enabled**           -- Returns the MySQL Server auto discovery enabled flag

    **xtrabackup_bin_path**             -- Returns the MySQL Server xtrabackup bin path

    **is_xtrabackup_enabled**           -- Returns the MySQL Server xtrabackup enabled flag

    **proxy_options**                   -- Returns the MySQL Server proxy options

    **mysql_enterprise_backup_binary_path** --  Returns the MySQL Enterprise backup binary path
    details

    **no_lock_status**                  --  Returns the No Lock check box status for MySQL Instance

    **ssl_enabled**                     --  Returns(boolean) True/False based on SSL status

"""

from __future__ import unicode_literals
from ..instance import Instance
from ..exception import SDKException
from ..credential_manager import Credential


class MYSQLInstance(Instance):
    """
    Class to represent a standalone MYSQL Instance
    """

    def __init__(self, agent_object, instance_name, instance_id=None):
        """Initialise the Subclient object.

            Args:
                agent_object    (object)  --  instance of the Agent class

                instance_name   (str)     --  name of the instance

                instance_id     (str)     --  id of the instance

                    default: None

            Returns:
                object - instance of the MYSQLInstance class

        """
        self._browse_restore_json = None
        self._commonoption_restore_json = None
        self._destination_restore_json = None
        self._fileoption_restore_json = None
        self._instance = None
        self.admin_option_json = None
        self.mysql_restore_json = None
        super(MYSQLInstance, self).__init__(agent_object, instance_name, instance_id)

    @property
    def port(self):
        """Returns the MySQL Server Port number.

        Returns:
            (str)   --  MySql server port number

        """
        return self._properties.get('mySqlInstance', {}).get('port', None)

    @property
    def mysql_username(self):
        """Returns the MySQL Server username.

        Returns:
            (str)   --  MySql server SA username

        """
        credential_name = self._properties.get('credentialEntity', {}).get('credentialName', None)
        return Credential(self._commcell_object, credential_name).credential_user_name

    @property
    def nt_username(self):
        """Returns the MySQL Server nt username.

        Returns:
            (str)   --  MySql server NT username

        """
        return self._properties.get('mySqlInstance', {}).get('NTUser', {}).get('userName', None)

    @property
    def config_file(self):
        """Returns the MySQL Server Config File location.

        Returns:
            (str)   --  MySql server config file location

        """
        return self._properties.get('mySqlInstance', {}).get('ConfigFile', None)

    @property
    def binary_directory(self):
        """Returns the MySQL Server Binary File location.

        Returns:
            (str)   --  MySql server binary directory

        """
        return self._properties.get('mySqlInstance', {}).get('BinaryDirectory', None)

    @property
    def version(self):
        """Returns the MySQL Server version number.

        Returns:
            (str)   --  MySql server version

        """
        return self._properties.get('mySqlInstance', {}).get('version', None)

    @property
    def log_data_directory(self):
        """Returns the MySQL Server log data directory.

        Returns:
            (str)   --  MySql server log directory path

        """
        return self._properties.get('mySqlInstance', {}).get('LogDataDirectory', None)

    @property
    def log_backup_sp_details(self):
        """Returns the MySQL Server Log backup SP details

        Returns:
            (dict)  --  MySql server log backup storage policy information

        """
        log_storage_policy_name = self._properties.get('mySqlInstance', {}).get(
            'logStoragePolicy', {}).get('storagePolicyName', None)
        log_storage_policy_id = self._properties.get('mySqlInstance', {}).get(
            'logStoragePolicy', {}).get('storagePolicyId', None)

        log_sp = {"storagePolicyName": log_storage_policy_name,
                  "storagePolicyId": log_storage_policy_id}
        return log_sp

    @property
    def command_line_sp_details(self):
        """Returns the MySQL Server commandline SP details.

        Returns:
            (dict)  --  MySql server commnadline storage policy information

        """
        cmd_storage_policy_name = self._properties.get('mySqlInstance', {}).get(
            'mysqlStorageDevice', {}).get('commandLineStoragePolicy', {}).get(
                'storagePolicyName', None)
        cmd_storage_policy_id = self._properties.get('mySqlInstance', {}).get(
            'mysqlStorageDevice', {}).get('commandLineStoragePolicy', {}).get(
                'storagePolicyId', None)

        command_sp = {"storagePolicyName": cmd_storage_policy_name,
                      "storagePolicyId": cmd_storage_policy_id}
        return command_sp

    @property
    def autodiscovery_enabled(self):
        """Returns the MySQL Server auto discovery enabled flag

        Returns:
            (bool)  --  True if auto discovery enabled
                        False if auto discovery not enabled

        """
        return self._properties.get('mySqlInstance', {}).get('EnableAutoDiscovery', False)

    @autodiscovery_enabled.setter
    def autodiscovery_enabled(self, value):
        """
        Sets the auto discovery attribute to True or False
        value (bool)   --  True or False

        """
        properties = self._properties
        update = {
            "EnableAutoDiscovery": value
        }
        properties['mySqlInstance'] = update
        self.update_properties(properties)

    @property
    def xtrabackup_bin_path(self):
        """Returns the MySQL Server xtrabackup bin path

        Returns:
            (str)   --  MySql server xtrabackup binary path

        """
        return self._properties.get(
            'mySqlInstance', {}).get(
                'xtraBackupSettings', {}).get('xtraBackupBinPath', "")

    @property
    def is_xtrabackup_enabled(self):
        """Returns the MySQL Server xtrabackup enabled flag

        Returns:
            (bool)  --  True if xtrabackup is enabled
                        False if xtrabackup is not enabled

        """
        return self._properties.get(
            'mySqlInstance', {}).get(
                'xtraBackupSettings', {}).get('enableXtraBackup', False)

    @property
    def proxy_options(self):
        """Returns the MySQL Server proxy options

        Returns:
            (dict)  --  MySql server proxy information

        """
        proxy_settings = self._properties.get('mySqlInstance', {}).get('proxySettings', {})
        proxy_opt = {
            "isUseSSL": proxy_settings.get('isUseSSL', False),
            "isProxyEnabled": proxy_settings.get('isProxyEnabled', False),
            "runBackupOnProxy": proxy_settings.get('runBackupOnProxy', False),
            "instanceId": proxy_settings.get('proxyInstance', {}).get('instanceId', None),
            "instanceName": proxy_settings.get('proxyInstance', {}).get('instanceName', None),
            "clientId": proxy_settings.get('proxyInstance', {}).get('clientId', None),
            "clientName": proxy_settings.get('proxyInstance', {}).get('clientName', None)}
        return proxy_opt

    @property
    def mysql_enterprise_backup_binary_path(self):
        """ Returns the MySQL Enterprise backup binary path detail

            Return Type: dict

        """
        meb_settings = self._properties.get('mySqlInstance', {}).get('mebSettings', {})
        return meb_settings

    @mysql_enterprise_backup_binary_path.setter
    def mysql_enterprise_backup_binary_path(self, value):
        """ Setter for MySQL Enterprise backup binary path

            Args:

                value (str)  -- Contains the MySQL Enterprise backup binary path to be updated
                in MySQL Instance property

        """
        if not isinstance(value, str):
            raise SDKException('Instance', '101')
        properties = self._properties
        meb_bin_path_update = {
            "enableMEB": False if value == '' else True,
            "mebBinPath": value
        }
        properties['mySqlInstance']['mebSettings'] = meb_bin_path_update
        self.update_properties(properties)

    @property
    def no_lock_status(self):
        """ Returns the status of No Lock Checkbox in MySQL Instance

            Returns:
            (bool)  --  True if No Lock checkbox is enabled
                        False if No Lock checkbox is disabled

        """
        return self._properties.get('mySqlInstance', {}).get('EnableNoLocking', False)

    @no_lock_status.setter
    def no_lock_status(self, value):
        """ Setter for No Lock property in MySQL Instance

            Args:

                value (bool)  -- True or False to enable or disable the No Lock
                property in MySQL Instance

        """
        if not isinstance(value, bool):
            raise SDKException('Instance', '101')
        properties = self._properties
        properties['mySqlInstance']['EnableNoLocking'] = value
        self.update_properties(properties)

    @property
    def ssl_enabled(self):
        """ Returns(boolean) True/False based on SSL status """
        return self._properties.get('mySqlInstance', {}).get('sslEnabled', False)

    def _get_instance_properties(self):
        """Gets the properties of this instance.

            Raises:
                SDKException:
                    if response is empty

                    if response is not success

        """
        super(MYSQLInstance, self)._get_instance_properties()
        self._instance = {
            "type": 0,
            "clientName": self._agent_object._client_object.client_name,
            "clientSidePackage": True,
            "subclientName": "",
            "backupsetName": "defaultDummyBackupSet",
            "instanceName": self.instance_name,
            "appName": self._agent_object.agent_name,
            "consumeLicense": True
        }

    def _restore_json(self, **kwargs):
        """Returns the JSON request to pass to the API as per the options selected by the user.

            Args:
                kwargs   (list)  --  list of options need to be set for restore

            Returns:
                dict - JSON request to pass to the API

        """
        rest_json = super(MYSQLInstance, self)._restore_json(**kwargs)
        restore_option = {}
        if kwargs.get("restore_option"):
            restore_option = kwargs["restore_option"]
            for key in kwargs:
                if not key == "restore_option":
                    restore_option[key] = kwargs[key]
        else:
            restore_option.update(kwargs)

        if restore_option["from_time"] is None:
            restore_option["from_time"] = {}

        if restore_option["to_time"] is None:
            restore_option["to_time"] = {}

        self._restore_admin_option_json(restore_option)
        self._restore_mysql_option_json(restore_option)
        rest_json["taskInfo"]["subTasks"][0]["options"]["restoreOptions"][
            "mySqlRstOption"] = self.mysql_restore_json
        rest_json["taskInfo"]["subTasks"][0]["options"]["adminOpts"] = self.admin_option_json
        return rest_json

    def restore_in_place(
            self,
            path=None,
            staging=None,
            dest_client_name=None,
            dest_instance_name=None,
            data_restore=True,
            log_restore=False,
            overwrite=True,
            copy_precedence=None,
            from_time=None,
            to_time=None,
            media_agent=None,
            table_level_restore=False,
            clone_env=False,
            clone_options=None,
            redirect_enabled=False,
            redirect_path=None,
            browse_jobid=None):
        """Restores the mysql data/log files specified in the input paths list to the same location.

            Args:
                path                    (list)  --  list of database/databases to be restored

                    default: None

                staging                 (str)   --  staging location for mysql logs during restores

                    default: None

                dest_client_name        (str)   --  destination client name where files are to be
                restored

                    default: None

                dest_instance_name      (str)   --  destination mysql instance name of destination
                client

                    default: None

                data_restore            (bool)  --  for data only/data+log restore

                    default: True

                log_restore             (bool)  --  for log only/data+log restore

                    default: False

                overwrite               (bool)  --  unconditional overwrite files during restore

                    default: True

                copy_precedence         (int)   --  copy precedence value of storage policy copy

                    default: None

                from_time               (str)   --  time to retore the contents after
                        format: YYYY-MM-DD HH:MM:SS

                    default: None

                to_time                 (str)   --  time to retore the contents before
                        format: YYYY-MM-DD HH:MM:SS

                    default: None

                media_agent             (str)   --  media agent associated

                    default: None

                table_level_restore     (bool)  --  Table level restore flag

                    default: False

                clone_env               (bool)  --  boolean to specify whether the database
                should be cloned or not

                    default: False

                clone_options           (dict)  --  clone restore options passed in a dict

                    default: None

                    Accepted format: {
                                        "stagingLocaion": "/gk_snap",
                                        "forceCleanup": True,
                                        "port": "5595",
                                        "libDirectory": "",
                                        "isInstanceSelected": True,
                                        "reservationPeriodS": 3600,
                                        "user": "",
                                        "binaryDirectory": "/usr/bin"

                                     }

                redirect_enabled         (bool)  --  boolean to specify if redirect restore is
                enabled

                    default: False

                redirect_path           (str)   --  Path specified in advanced restore options
                in order to perform redirect restore

                    default: None

                browse_jobid           (int)   --  Browse jobid to browse and restore from

                    default: None

            Returns:
                object - instance of the Job class for this restore job

            Raises:
                SDKException:
                    if paths is not a list

                    if failed to initialize job

                    if response is empty

                    if response is not success

        """
        if not (isinstance(path, list) and
                isinstance(overwrite, bool)):
            raise SDKException('Instance', '101')

        if path == []:
            raise SDKException('Instance', '104')

        if dest_client_name is None:
            dest_client_name = self._agent_object._client_object.client_name

        if dest_instance_name is None:
            dest_instance_name = self.instance_name

        request_json = self._restore_json(
            paths=path,
            staging=staging,
            dest_client_name=dest_client_name,
            dest_instance_name=dest_instance_name,
            data_restore=data_restore,
            log_restore=log_restore,
            overwrite=overwrite,
            copy_precedence=copy_precedence,
            from_time=from_time,
            to_time=to_time,
            media_agent=media_agent,
            table_level_restore=table_level_restore,
            clone_env=clone_env,
            clone_options=clone_options,
            redirect_enabled=redirect_enabled,
            redirect_path=redirect_path,
            browse_jobid=browse_jobid)

        return self._process_restore_response(request_json)

    def restore_out_of_place(
            self,
            path=None,
            staging=None,
            dest_client_name=None,
            dest_instance_name=None,
            data_restore=True,
            log_restore=False,
            overwrite=True,
            copy_precedence=None,
            from_time=None,
            to_time=None,
            media_agent=None,
            table_level_restore=False,
            clone_env=False,
            clone_options=None,
            redirect_enabled=False,
            redirect_path=None,
            browse_jobid=None,
            recurringRestore=False):
        """
        Method to perform out of place restore of MySQL data/log/recurring files to the destination client location.

            Args:
                path                    (list)  --  list of database/databases to be restored
                staging                 (str)   --  staging location for mysql logs during restores
                dest_client_name        (str)   --  destination client name where files are to be restored
                dest_instance_name      (str)   --  destination mysql instance name of destination client
                data_restore            (bool)  --  for data only/data+log restore
                log_restore             (bool)  --  for log only/data+log restore
                overwrite               (bool)  --  unconditional overwrite files during restore
                copy_precedence         (int)   --  copy precedence value of storage policy copy
                from_time               (str)   --  time to restore the contents after
                        format: YYYY-MM-DD HH:MM:SS
                to_time                 (str)   --  time to restore the contents before
                        format: YYYY-MM-DD HH:MM:SS
                media_agent             (str)   --  media agent associated
                table_level_restore     (bool)  --  Table level restore flag
                clone_env               (bool)  --  boolean to specify whether the database should be cloned or not
                clone_options           (dict)  --  clone restore options passed in a dict
                redirect_enabled         (bool)  --  boolean to specify if redirect restore is enabled
                redirect_path           (str)   --  Path specified in advanced restore options
                in order to perform redirect restore
                browse_jobid           (int)   --  Browse jobid to browse and restore from
                recurringRestore       (bool)  --  for Recurring restore

            Returns:
                object - instance of the Job class for this restore job

            Raises:
                SDKException:
                    if paths is not a list

                    if failed to initialize job

                    if destination client name is empty

                    if destination Instance name empty

                    if response is empty

                    if response is not success
        """
        if not (isinstance(path, list) and
                isinstance(overwrite, bool)):
            raise SDKException('Instance', '101')

        if path == []:
            raise SDKException('Instance', '104')

        if dest_client_name is None:
            raise SDKException('Client', '102',
                               "The destination client name is missing. "
                               "Please provide a valid destination client name to proceed")

        if dest_instance_name is None:
            raise SDKException('Instance', '102',
                               "The destination Instance name is missing. "
                               "Please provide a valid destination Instance name to proceed")

        request_json = self._restore_json(
            paths=path,
            staging=staging,
            dest_client_name=dest_client_name,
            dest_instance_name=dest_instance_name,
            data_restore=data_restore,
            log_restore=log_restore,
            overwrite=overwrite,
            copy_precedence=copy_precedence,
            from_time=from_time,
            to_time=to_time,
            media_agent=media_agent,
            table_level_restore=table_level_restore,
            clone_env=clone_env,
            clone_options=clone_options,
            redirect_enabled=redirect_enabled,
            redirect_path=redirect_path,
            browse_jobid=browse_jobid,
            recurringRestore=recurringRestore)

        return self._process_restore_response(request_json)

    def _restore_browse_option_json(self, value):
        """setter for the Browse options for restore in Json"""

        if not isinstance(value, dict):
            raise SDKException('Instance', '101')
        super(MYSQLInstance, self)._restore_browse_option_json(value)
        self._browse_restore_json['backupset'] = {
            "clientName": self._agent_object._client_object.client_name,
            "backupsetName": "defaultDummyBackupSet"
        }

        if value.get("browse_jobid"):
            self._browse_restore_json['browseJobId'] = value.get("browse_jobid")

        if value.get("from_time") and value.get("to_time"):
            self._browse_restore_json["timeRange"] = {"fromTime" : value.get("from_time"),
                                                      "toTime" : value.get("to_time")}

    def _restore_common_options_json(self, value):
        """setter for the Common options in restore JSON"""

        if not isinstance(value, dict):
            raise SDKException('Instance', '101')

        self._commonoption_restore_json = {
            "restoreToDisk": False,
            "onePassRestore": False,
            "revert": False,
            "syncRestore": False
        }

    def _restore_destination_json(self, value):
        """setter for the MySQL Destination options in restore JSON"""

        if not isinstance(value, dict):
            raise SDKException('Instance', '101')

        self._destination_restore_json = {
            "destinationInstance": {
                "clientName": value.get("dest_client_name", ""),
                "instanceName": value.get("dest_instance_name", ""),
                "appName": "MySQL"
            },
            "destClient": {
                "clientName": value.get("dest_client_name", "")
            }
        }

    def _restore_fileoption_json(self, value):
        """setter for the fileoption restore option in restore JSON"""

        if not isinstance(value, dict):
            raise SDKException('Instance', '101')

        self._fileoption_restore_json = {
            "sourceItem": value.get("paths", [])
        }

    def _restore_admin_option_json(self, value):
        """setter for the admin restore option in restore JSON"""

        if not isinstance(value, dict):
            raise SDKException('Instance', '101')

        self.admin_option_json = {
            "contentIndexingOption": {
                "subClientBasedAnalytics": False
            }
        }

    def _restore_mysql_option_json(self, value):
        """setter for the mysql restore option in restore JSON"""

        if not isinstance(value, dict):
            raise SDKException('Instance', '101')

        self.mysql_restore_json = {
            "destinationFolder": "",
            "data": value.get("data_restore", True),
            "log": value.get("log_restore", True),
            "recurringRestore": value.get("recurringRestore", False),
            "temporaryStagingLocation": value.get("staging", ""),
            "dataStagingLocation": "",
            "logRestoreType": 0,
            "tableLevelRestore": value.get("table_level_restore", False),
            "pointofTime": True if value.get("to_time") else False,
            "instanceRestore": True,
            "isCloneRestore": value.get("clone_env", False),
            "fromTime": value.get("from_time", {}),
            "refTime": value.get("to_time", {}),
            "destinationServer": {
                "name": ""
            }
        }
        if value.get("table_level_restore"):
            self.mysql_restore_json['dropTable'] = True
            self.mysql_restore_json['instanceRestore'] = False

        if value.get("clone_env", False):
            self.mysql_restore_json["cloneOptions"] = value.get("clone_options", "")

        if value.get("redirect_path"):
            self.mysql_restore_json["redirectEnabled"] = True
            self.mysql_restore_json["redirectItems"] = [value.get("redirect_path")]

        if value.get("from_time"):
            self.mysql_restore_json["fromTime"] = {"time" : value.get("to_time")}

        if value.get("to_time"):
            self.mysql_restore_json["refTime"] = {"time" : value.get("to_time")}

        if value.get("to_time"):
            self.mysql_restore_json["pointInTime"] = {"time" : value.get("to_time")}

        if value.get("dest_instance_name"):
            self.mysql_restore_json["destinationServer"] = {"name": value.get(
                "dest_instance_name")}



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------------
# Copyright Commvault Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# --------------------------------------------------------------------------

"""File for operating on a Azure CosmosDB Subclient.


AzureCosmosDBSubclient:  Derived class from CloudAppsSubclient Base class, representing a
                        Azure Cosmos DB subclient(Azure CosmosDB Cassandra API, ..), and
                        to perform operations on that subclient

AzureCosmosDBSubclient:

    _get_subclient_properties()         --  gets the properties of Cloud Database Subclient

"""
from ..casubclient import CloudAppsSubclient
from ...exception import SDKException


class AzureCosmosDBSubclient(CloudAppsSubclient):
    """ Derived class from Subclient Base class, representing a Azure Cosmos DB subclient,
            and to perform operations on that subclient. """

    def _get_subclient_properties(self):
        """ Gets the subclient related properties of Cloud Database subclient. """

        super(AzureCosmosDBSubclient, self)._get_subclient_properties()





# Copyright Commvault Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# --------------------------------------------------------------------------

"""Main file for performing threat/file type analysis on clients/laptops

TAServers , TAServer, AnomalyType are the three classes defined in this file

TAServers - class to represent all servers in threat indicators

TAServer  - class to represent single server in threat indicators

AnomalyType - class to represent different types of anomaly

TAServers:

    __init__()                          --  initialise object of the TAServers class

    _get_threat_indicators()            --  returns the list of threat indicators client for this CS

    _response_not_success()             --  parses through the exception response, and raises SDKException

    refresh()                           --  refreshes the threat indicators servers for this CS

    has()                               --  Checks whether given server name exists in threat indicators or not

    get()                               --  returns the server class object for given server name

    run_scan()                          --  runs anomaly scan on given server

TAServer:

    __init__()                          --  initialise object of the TAServer class

    _get_anomalies_stats()              --  returns the anomalies stats for this client

    _get_anomaly_records()              --  returns list containing files anomaly record details of this client

    _response_not_success()             --  parses through the exception response, and raises SDKException

    refresh()                           --  refreshes the server anomalies

    clear_anomaly()                     --  clears the anomalies present for this client

Server Attributes:

    **anomaly_records**             --  returns the list of anomaly records for this client

    **threat_anomaly_stats**        --  returns the dict of threat anomalies stats for this client

    **anomaly_stats**               --  returns the dict of file types/data anomalies stats for this client

    **datasource_id**               --  associated data source id for threat scan / analysis server

    **anomaly_file_count**          --  returns the total anomaly file count for this client


"""
import copy
import datetime
import enum
import time

from ..exception import SDKException
from ..monitoringapps.constants import ThreatConstants, FileTypeConstants, RequestConstants


class AnomalyType(enum.Enum):
    """Enum class for Anomaly type"""
    FILE_ACTIVITY = 16
    FILE_TYPE = 32
    THREAT_ANALYSIS = 64
    FILE_DATA = 128


class TAServers():
    """class to represent all servers in threat indicators"""

    def __init__(self, commcell_object):
        """Initializes an instance of the TAServers class.

            Args:

                commcell_object     (object)    --  instance of the commcell class

            Returns:

                object  -   instance of the Servers class

        """
        self._commcell_object = commcell_object
        self._cvpysdk_object = commcell_object._cvpysdk_object
        self._services = commcell_object._services
        self._threat_indicators = []
        self._servers = []
        self._API_GET_ALL_INDICATORS = self._services['GET_THREAT_INDICATORS']
        self._API_RUN_SCAN = self._services['RUN_ANOMALY_SCAN']
        self.refresh()

    def _response_not_success(self, response):
        """Helper function to raise an exception when reponse status is not 200 (OK).

            Args:
                response    (object)    --  response class object,

                received upon running an API request, using the `requests` python package

        """
        raise SDKException('Response', '101', self._commcell_object._update_response_(response.text))

    def _get_threat_indicators(self):
        """returns the list of threat indicators for this CS

            Args:

                None

            Returns:

                list(dict)      - server threat details

                        Eg:-

                                {
                                  "anomalyType": 8,
                                  "modCount": 13705,
                                  "renameCount": 1,
                                  "isVMeSupported": true,
                                  "refTime": 1727519811,
                                  "deleteCount": 73280,
                                  "createCount": 19,
                                  "location": "",
                                  "osInfo": {
                                    "osInfo": {
                                      "Type": "Windows",
                                      "SubType": "Server",
                                      "osId": 210,
                                      "OsDisplayInfo": {
                                        "ProcessorType": "WinX64",
                                        "OSName": "Windows Server 2019 Datacenter"
                                      }
                                    }
                                  },
                                  "client": {
                                    "clientId": 9,
                                    "clientName": "xx",
                                    "displayName": "xx_dn"
                                  }
                                }
        """
        flag, response = self._cvpysdk_object.make_request('GET', self._API_GET_ALL_INDICATORS)
        _threat_indicators = []
        if flag:
            if response.json() and 'anomalyClients' in response.json():
                _threat_indicators = response.json()['anomalyClients']
                for _client in _threat_indicators:
                    if 'client' in _client:
                        _display_name = _client['client'].get('displayName', '')
                        self._servers.append(_display_name.lower())
            elif bool(response.json()):
                raise SDKException('ThreatIndicators', '103')
            return _threat_indicators
        self._response_not_success(response)

    def has(self, name):
        """Checks whether given server name exists in threat indicators or not

            Args:

                name        (str)       --  Name of the server

            Returns:

                bool    --  True if server name exists in threat indicators

        """
        if name.lower() in self._servers:
            return True
        return False

    def run_scan(
            self,
            server_name,
            anomaly_types,
            index_server_name=None,
            storage_pool=None,
            from_time=None,
            to_time=None):
        """runs anomaly scan on given server name

            Args:

                server_name     (str)           --  Server name to analyze

                anomaly_types    (list)         --  list of anomaly to analyze on client

                index_server_name   (str)       --  Index server name to be used for scan

                storage_pool        (str)       --  Storage pool name to be used for scan

                from_time           (int)       --  epoch timestamp from when scan will analyze in backup

                to_time             (int)       --  epoch timestamp to which scan will analyze in backup

            Returns:

                int --  job id of scan job launched

            Raises:

                SDKException:

                    if failed to start job

                    if input data type is not valid

        """
        if not isinstance(server_name, str):
            raise SDKException('ThreatIndicators', '101')
        if not isinstance(anomaly_types, list):
            raise SDKException('ThreatIndicators', '101')
        if not self._commcell_object.clients.has_client(server_name):
            raise SDKException('ThreatIndicators', '102', 'Given server is not found in CS')
        req_json = copy.deepcopy(RequestConstants.RUN_SCAN_JSON)
        req_json['client']['clientId'] = int(self._commcell_object.clients.get(server_name).client_id)
        ta_flag = 0
        for each_anomaly in anomaly_types:
            if each_anomaly.name == AnomalyType.FILE_DATA.name:
                ta_flag = ta_flag + 2
            elif each_anomaly.name == AnomalyType.THREAT_ANALYSIS.name:
                ta_flag = ta_flag + 1
        req_json['threatAnalysisFlags'] = int(ta_flag)
        if not from_time and not to_time:
            to_time = int(time.time())
            req_json['timeRange']['toTime'] = int(to_time)
            to_time = datetime.datetime.fromtimestamp(to_time)
            from_time = to_time - datetime.timedelta(days=7)
            req_json['timeRange']['fromTime'] = int(from_time.timestamp())
        else:
            req_json['timeRange']['toTime'] = to_time
            req_json['timeRange']['fromTime'] = from_time

        if int(self._commcell_object.commserv_oem_id) == 119:
            if not storage_pool:
                raise SDKException('ThreatIndicators', '107')
            req_json.pop('indexServer')
            spool_obj = self._commcell_object.storage_pools.get(storage_pool)
            req_json['backupDetails'][0]['copyId'] = int(spool_obj.copy_id)
            req_json['backupDetails'][0]['storagePoolId'] = int(spool_obj.storage_pool_id)
        else:
            req_json.pop('backupDetails')
            if not index_server_name:
                raise SDKException('ThreatIndicators', '108')
            req_json['indexServer']['clientId'] = int(
                self._commcell_object.index_servers.get(index_server_name).index_server_client_id)

        flag, response = self._commcell_object._cvpysdk_object.make_request(
            'POST', self._API_RUN_SCAN, req_json
        )
        if flag:
            if response.json() and 'jobId' in response.json():
                return response.json()['jobId']
            raise SDKException('ThreatIndicators', '109')
        self._response_not_success(response)

    def get(self, server_name):
        """returns Server class object for given server name

                Args:

                    server_name        (str)       --  client name

                Returns:

                    obj --  Instance of Server class

                Raises:

                    SDKException:

                            if failed to find given server

                            if input is not valid

        """
        if not isinstance(server_name, str):
            raise SDKException('ThreatIndicators', '101')
        if not self.has(server_name):
            raise SDKException('ThreatIndicators', '105')
        return TAServer(commcell_object=self._commcell_object, server_name=server_name)

    def refresh(self):
        """Refresh the threat indicator servers associated with CS"""
        self._servers = []
        self._threat_indicators = self._get_threat_indicators()


class TAServer:

    def __init__(self, commcell_object, server_name):
        """Initializes an instance of the TAServer class.

            Args:

                commcell_object     (object)    --  instance of the commcell class

                server_name         (str)       --  Name of the server


            Returns:

                object  -   instance of the Server class

        """
        self._commcell_object = commcell_object
        self._update_response_ = commcell_object._update_response_
        self._cvpysdk_object = commcell_object._cvpysdk_object
        self._services = commcell_object._services
        self._server_name = server_name
        self._server_id = self._commcell_object.clients.get(server_name).client_id
        self._anomaly_records = None
        self._threat_anomaly_stats = None
        self._anomaly_stats = None
        self._threat_dsid = None
        self._API_GET_ALL_INDICATORS = self._services['GET_THREAT_INDICATORS']
        self._API_GET_ALL_ANOMALIES = self._services['GET_ALL_CLIENT_ANOMALIES']
        self._API_CLEAR_ANOMALIES = self._services['CLEAR_ANOMALIES']
        self.refresh()

    def _response_not_success(self, response):
        """Helper function to raise an exception when reponse status is not 200 (OK).

            Args:
                response    (object)    --  response class object,

                received upon running an API request, using the `requests` python package

        """
        raise SDKException('Response', '101', self._commcell_object._update_response_(response.text))

    def _get_anomalies_stats(self):
        """returns the anomalies stats for this client"""
        flag, response = self._cvpysdk_object.make_request('GET', self._API_GET_ALL_INDICATORS)
        threat_stats = {}
        stats = {}
        if flag:
            if response.json() and 'anomalyClients' in response.json():
                _resp = response.json()['anomalyClients']
                for _client in _resp:
                    if 'client' in _client:
                        if _client['client'].get('displayName', '').lower() == self._server_name.lower():
                            threat_stats[ThreatConstants.FIELD_INFECTED_COUNT] = _client.get(
                                ThreatConstants.FIELD_INFECTED_COUNT, 0)
                            threat_stats[ThreatConstants.FIELD_FINGERPRINT_COUNT] = _client.get(
                                ThreatConstants.FIELD_FINGERPRINT_COUNT, 0)
                            stats[FileTypeConstants.FIELD_CREATE_COUNT] = _client.get(
                                FileTypeConstants.FIELD_CREATE_COUNT, 0)
                            stats[FileTypeConstants.FIELD_DELETE_COUNT] = _client.get(
                                FileTypeConstants.FIELD_DELETE_COUNT, 0)
                            stats[FileTypeConstants.FIELD_MODIFIED_COUNT] = _client.get(
                                FileTypeConstants.FIELD_MODIFIED_COUNT, 0)
                            stats[FileTypeConstants.FIELD_RENAME_COUNT] = _client.get(
                                FileTypeConstants.FIELD_RENAME_COUNT, 0)
                            self._threat_dsid = _client.get('dataSourceId', 0)
            elif bool(response.json()):
                raise SDKException('ThreatIndicators', '103')
            return stats, threat_stats
        self._response_not_success(response)

    def _get_anamoly_records(self):
        """returns file type anomaly records for this client"""
        api = self._API_GET_ALL_ANOMALIES % (0, self._server_id)  # filter=0 to fetch all anomalies types
        flag, response = self._cvpysdk_object.make_request('GET', api)
        if flag:
            if response.json() and 'clientInfo' in response.json():
                _resp = response.json()['clientInfo'][0]
                if 'anomalyRecordList' in _resp:
                    return _resp['anomalyRecordList']
                else:
                    # for fingerprint analysis, record list will be empty
                    return []
            raise SDKException('ThreatIndicators', '104')
        self._response_not_success(response)

    def refresh(self):
        """Refresh the anomalies details associated with this server"""
        self._threat_dsid = 0
        self._anomaly_records = self._get_anamoly_records()
        self._anomaly_stats, self._threat_anomaly_stats = self._get_anomalies_stats()

    def clear_anomaly(self, anomaly_types):
        """clears the anomalies for this server

            Args:

                anomaly_types       (list)      --  list of anomalies to clear (Refer to AnomalyType class)

            Returns:

                None

            Raises:

                SDKException:

                    if failed to clear anomaly

                    if input is not valid

        """
        if not isinstance(anomaly_types, list):
            raise SDKException('ThreatIndicators', '101')
        anomalies_to_clear = []
        for each_anomaly in anomaly_types:
            anomalies_to_clear.append(each_anomaly.name)
        _req_json = copy.deepcopy(RequestConstants.CLEAR_ANOMALY_JSON)
        _req_json['clients'][0]['clientId'] = int(self._server_id)
        _req_json['clients'][0]['displayName'] = self._server_name
        _req_json['anomalyTypes'] = anomalies_to_clear
        if AnomalyType.FILE_DATA.name in anomalies_to_clear or AnomalyType.THREAT_ANALYSIS.name in anomalies_to_clear:
            _req_json['clients'][0]['dataSourceId'] = self.datasource_id
        flag, response = self._commcell_object._cvpysdk_object.make_request(
            'POST', self._API_CLEAR_ANOMALIES, _req_json
        )
        if flag:
            if response.json() and 'error' in response.json():
                response = response.json()['error']
                if 'errorCode' in response and response['errorCode'] != 0:
                    raise SDKException(
                        'ThreatIndicators',
                        '106')
                elif 'errorCode' not in response:
                    raise SDKException(
                        'ThreatIndicators',
                        '102',
                        f'Something went wrong during clear anomaly - {response.json()}')
                self.refresh()
                return
        self._response_not_success(response)

    @property
    def anomaly_records(self):
        """returns the anomaly records list

            Returns:

                list --  anomaly records for this client

        """
        return self._anomaly_records

    @property
    def threat_anomaly_stats(self):
        """returns the threat anomaly stats

            Returns:

                dict --  threat anomaly stats for this client

        """
        return self._threat_anomaly_stats

    @property
    def anomaly_stats(self):
        """returns the file type/data anomaly stats

            Returns:

                dict --  file/data anomaly stats for this client

        """
        return self._anomaly_stats

    @property
    def datasource_id(self):
        """returns the threat datasource_id associated with this server

            Returns:

                int --  datasource id

        """
        return int(self._threat_dsid)

    @property
    def anomaly_file_count(self):
        """returns the total anomalies file count for this server

            Returns:

                int --  anomaly file count

        """
        _total_files = 0
        _total_files = _total_files + sum(self._anomaly_stats.values())
        _total_files = _total_files + sum(self._threat_anomaly_stats.values())
        return _total_files
